
import unittest
from wisecon.stock.tick.tick import Tick


class TestTick(unittest.TestCase):

    def test_tick(self):
        data = Tick(code="301618", verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_tick_1(self):
        data = Tick(code="300059", verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_tick_2(self):
        data = Tick(code="002249", verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())



from typing import Any, Dict, Callable, Optional, Literal, Annotated
from wisecon.types import BaseMapping, APIStockTrends2


__all__ = [
    "KlineMinMapping",
    "KlineMin",
]


class KlineMinMapping(BaseMapping):
    """字段映射 股票-KlineMin"""
    columns: Dict = {
        "time": "时间",
        "open": "开盘",
        "close": "收盘",
        "high": "最高",
        "low": "最低",
        "volume": "成交量",
        "turnover": "成交额",
        "unknown": "unknown",
    }


class KlineMin(APIStockTrends2):
    """查询 股票-KlineMin，可以获取最大近5天的分钟级K线数据"""
    def __init__(
            self,
            security_code: Annotated[Optional[str], "证券代码", False] = None,
            plate_code: Annotated[Optional[str], "板块代码", False] = None,
            n_days: Annotated[Optional[int], "数据时限", False] = 1,
            verbose: Annotated[Optional[bool], "是否打印日志", False] = False,
            logger: Annotated[Optional[Callable], "日志对象", False] = None,
            **kwargs: Annotated[Any, "其他参数", False]
    ):
        """
        查询股票1分钟级的K线数据

        Notes:
            ```python
            from wisecon.stock.kline import *

            # 1. 查询股票的 K线数据
            data = KlineMin(security_code="300069", n_days=1).load()
            data.to_frame(chinese_column=True)

            # 2. 查询板块的 K线数据
            data = KlineMin(plate_code="BK0887", n_days=1).load()
            print(data.to_markdown(chinese_column=True))
            ```

        Args:
            security_code: 股票代码
            plate_code: 板块代码
            n_days: 查询天数
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.plate_code = plate_code
        self.n_days = n_days
        self.mapping = KlineMinMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="股票-KlineMin")
        self.validate_n_days()

    def validate_n_days(self):
        """"""
        if self.n_days > 5:
            raise ValueError("n_days should be less than or equal to 5")

    def params_secid(self) -> str:
        """"""
        if self.security_code:
            return f"0.{self.security_code}"
        elif self.plate_code:
            return f"90.{self.plate_code}"

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "ndays": self.n_days,
            "secid": self.params_secid(),
        }
        return self.base_param(update=params)



import json
import requests
from functools import partial
from typing import Any, List, Dict, Callable, Optional
from wisecon.types import BaseMapping, ResponseData
from wisecon.utils import headers, LoggerMixin, jquery_mock_callback, time2int, filter_dict_by_key


__all__ = [
    "FundHistMapping",
    "FundHist",
]


class FundHistMapping(BaseMapping):
    """"""
    columns: Dict = {
        "FSRQ": "净值日期",
        "DWJZ": "单位净值",
        "LJJZ": "累计净值",
        "JZZZL": "日增长率",
        "SGZT": "申购状态",
        "SHZT": "赎回状态",
    }


class FundHist(LoggerMixin):
    """ Fund History """
    def __init__(
            self,
            fund_code: str,
            start_date: Optional[str] = "",
            end_date: Optional[str] = "",
            limit: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """"""
        self.fund_code = fund_code
        self.start_date = start_date
        self.end_date = end_date
        self.limit = limit
        self.mapping = FundHistMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

    def _base_url(self) -> str:
        """
        https://fund.eastmoney.com/data/FundGuideapi.aspx?
        dt=4
        sd=
        ed=
        tp=BK000174
        se>5
        sc=2n
        st=desc
        pi=1
        pn=27
        zf=diy
        sh=list
        rnd=0.13975355548916668


        :return:
        """
        base_url = "https://api.fund.eastmoney.com/f10/lsjz"
        return base_url

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "callback": jquery_mock_callback(),
            "fundCode": self.fund_code,
            "pageIndex": "1",
            "pageSize": self.limit,
            "startDate": self.start_date,
            "endDate": self.end_date,
            "_": time2int(),
        }
        return params

    def request_json(self) -> Dict:
        """
        :return:
        """
        headers.update({'Referer': 'https://fundf10.eastmoney.com/'})
        response = requests.get(self._base_url(), params=self.to_params(), headers=headers)
        content = response.text
        content = content[content.find("(") + 1: content.rfind(")")]
        data = json.loads(content)
        return data

    def clean_data(self, data: List[Dict]) -> List[Dict]:
        """"""
        _filter_dict = partial(filter_dict_by_key, keys=list(self.mapping.columns.keys()))
        data = list(map(_filter_dict, data))
        return data

    def load_data(self) -> ResponseData:
        """
        :return:
        """
        metadata = self.request_json()
        data = metadata.pop("Data")
        data = self.clean_data(data.get("LSJZList"))
        self.update_metadata(metadata)
        return ResponseData(data=data, metadata=metadata)

    def update_metadata(self, metadata: Dict):
        """"""
        columns = self.mapping.filter_columns(columns=self.mapping.columns)
        metadata.update({
            "description": "基金历史净值",
            "columns": columns,
        })



from typing import Any, Dict, Optional, Callable, Literal
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "InstitutionSeatMapping",
    "InstitutionSeat",
]


class InstitutionSeatMapping(BaseMapping):
    """字段映射 机构席位追踪"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "AMOUNT": "金额",
        "ONLIST_TIMES": "上榜次数",
        "BUY_AMT": "买入金额",
        "BUY_TIMES": "买入次数",
        "SELL_AMT": "卖出金额",
        "SELL_TIMES": "卖出次数",
        "NET_BUY_AMT": "净买入金额",
        "M1_CLOSE_ADJCHRATE": "1个月收盘调整幅度",
        "M3_CLOSE_ADJCHRATE": "3个月收盘调整幅度",
        "M6_CLOSE_ADJCHRATE": "6个月收盘调整幅度",
        "Y1_CLOSE_ADJCHRATE": "1年收盘调整幅度",
        "SECURITY_CODE": "证券代码",
        "CLOSE_PRICE": "收盘价格",
        "CHANGE_RATE": "变动率",
        "STATISTICSCYCLE": "统计周期",
        "BOARD_NAME": "板块名称",
        "BOARD_CODE": "板块代码",
        "SECURITY_TYPE_CODE": "证券类型代码"
    }


class InstitutionSeat(APIDataV1RequestData):
    """查询 机构席位追踪"""

    def __init__(
            self,
            statistics_cycle: Literal["1m", "3m", "6m", "12m"] = "1m",
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.ranking import *

            data = InstitutionSeat(statistics_cycle="1m").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            statistics_cycle: 统计周期, 可选值: 1m, 3m, 6m, 12m, 默认值: 1m
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.statistics_cycle = statistics_cycle
        self.size = size
        self.mapping = InstitutionSeatMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="机构席位追踪",)
        self.conditions = []

    def params_statistics_cycle(self):
        """"""
        cycle_mapping = {"1m": "01", "3m": "02", "6m": "03", "12m": "04"}
        self.conditions.append(f'(STATISTICSCYCLE="{cycle_mapping.get(self.statistics_cycle, "01")}")')

    def params_filter(self) -> str:
        """"""
        self.params_statistics_cycle()
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "ONLIST_TIMES,SECURITY_CODE",
            "sortTypes": "-1,1",
            "reportName": "RPT_ORGANIZATION_SEATNEW",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "TaxMapping",
    "Tax",
]


class TaxMapping(BaseMapping):
    """"""
    columns: Dict = {
        "START_DATE": "开始时间",
        "END_DATE": "截至时间",
        "TAX_INCOME": "税收收入合计(亿元)",
        "TAX_INCOME_SAME": "较上年同期(%)",
        "TAX_INCOME_SEQUENTIAL": "季度环比(%)",
    }

    
class Tax(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = TaxMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 全国税收收入")
    
    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "TAX_INCOME", "TAX_INCOME_SAME", "TAX_INCOME_SEQUENTIAL"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_TAX",
        }
        return self.base_param(update=params)



import unittest
from wisecon.macro import *


class TestCPI(unittest.TestCase):
    """"""

    def test_asset_invest(self):
        data = AssetInvest(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_boom_index(self):
        """"""
        data = BoomIndex(size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_cpi(self):
        data = CPI(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_currency_supply(self):
        """"""
        data = CurrencySupply(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_customs(self):
        """"""
        data = Customs(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_deposit_rate(self):
        """"""
        data = DepositRate(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_deposit_reserve(self):
        """"""
        data = DepositReserve(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_faith_index(self):
        """"""
        data = FaithIndex(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_fdi(self):
        """"""
        data = FDI(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_forex_deposit(self):
        """"""
        data = ForexDeposit(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_forex_loan(self):
        """"""
        data = ForexLoan(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_gdp(self):
        """"""
        data = GDP(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_gold_currency(self):
        """"""
        data = GoldCurrency(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_goods_index(self):
        """"""
        data = GoodsIndex(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_gov_income(self):
        """"""
        data = GovIncome(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_old_hose(self):
        """"""
        data = HoseIndexOld(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_new_hose(self):
        """"""
        data = HoseIndexNew(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

        data = HoseIndexNew(size=10, cities=["杭州", "深圳"]).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

        data = HoseIndexNew(size=10, report_date="2023-01-01").load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_imp_interest(self):
        """"""
        data = ImpInterest(size=10, ).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_indus_grow(self):
        """"""
        data = IndusGrow(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_lpr(self):
        """"""
        data = LPR(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_new_loan(self):
        """"""
        data = NewLoan(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_oil_price(self):
        """"""
        data = OilPrice(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_pmi(self):
        """"""
        data = PMI(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_ppi(self):
        """"""
        data = PPI(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_stock_open(self):
        """"""
        data = StockOpen(size=10).load()
        data.show_columns()
        print(data.to_markdown(chinese_column=True))

    def test_stock_statistics(self):
        """"""
        data = StockStatistics(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_tax(self):
        """"""
        data = Tax(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_total_retail(self):
        """"""
        data = TotalRetail(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))

    def test_transfer_fund(self):
        """"""
        data = TransferFund(size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True))


class TestLoopLoadData(unittest.TestCase):
    """"""
    def test_loop(self):
        """"""
        macro_mapping = {
            "居民消费价格指数(CPI)": CPI,
            "工业品出厂价格指数(PPI)": PPI,
            "国内生产总值(GDP)": GDP,
            "采购经理人指数(PMI)": PMI,
            "城镇固定资产投资": AssetInvest,
            "房价指数(old)": HoseIndexOld,
            "新房价指数": HoseIndexNew,
            "企业景气及企业家信心指数": BoomIndex,
            "工业增加值增长": IndusGrow,
            "企业商品价格指数": GoodsIndex,
            "消费者信心指数": FaithIndex,
            "社会消费品零售总额": TotalRetail,
            "货币供应量": CurrencySupply,
            "海关进出口增减情况一览表": Customs,
            "全国股票交易统计表": StockStatistics,
            "外汇和黄金储备": GoldCurrency,
            "交易结算资金(银证转账)": TransferFund,
            "股票账户统计表(新)": StockOpen,
            "外商直接投资数据(FDI)": FDI,
            "财政收入": GovIncome,
            "全国税收收入": Tax,
            "新增信贷数据": NewLoan,
            "银行间拆借利率": ImpInterest,
            "本外币存款": ForexDeposit,
            "外汇贷款数据": ForexLoan,
            "存款准备金率": DepositReserve,
            "利率调整": DepositRate,
            "油价": OilPrice,
            "LPR数据": LPR,
        }
        for macro_name, macro_class in macro_mapping.items():
            print(macro_name)
            data = macro_class(size=5).load()
            data.show_columns()
            print(data.to_frame(chinese_column=True).to_markdown())
            print("\n\n")



import pprint
import pandas as pd
from pydantic import BaseModel, Field
from typing import Any, List, Dict, Literal, Optional


__all__ = [
    "Metadata",
    "ResponseData",
]


class Metadata(BaseModel):
    """"""
    columns: Dict[str, Any] = Field(default={})
    description: str = Field(default="")
    response: Dict[str, Any] = Field(default={})

    def __init__(self, **kwargs: Any):
        """元数据

        Args:
            **kwargs: 参数
        """
        super().__init__(**kwargs)
        self.columns = {k: v for k, v in self.columns.items() if v}


class ResponseData(BaseModel):
    """请求返回数据的数据类"""
    metadata: Optional[Metadata] = Field(default=None)
    data: List[Dict] = Field(default=[])

    def __init__(
            self,
            data: List[Dict],
            metadata: Optional[Metadata] = None,
            **kwargs: Any,
    ):
        """封装请求返回数据方法类

        Args:
            data: 返回数据
            metadata: 元数据
            **kwargs: 其他参数
        """
        super().__init__(**kwargs)
        self.data = data
        self.metadata = metadata

    def show_columns(self):
        """展示数据的字段信息"""
        pprint.pprint(self.metadata.columns, indent=4)

    def _trans_chinese_columns(self, item: Dict) -> Dict:
        """将数据转换为中文名

        Args:
            item: 一条数据

        Returns:
            Dict: 转换后的数据
        """
        return {self.metadata.columns.get(key, key): value for key, value in item.items()}

    def drop_no_zh_columns(self):
        """删除非中文字段"""
        exits_columns = [k for k, v in self.metadata.columns.items() if v]
        self.data = [{k: v for k, v in item.items() if k in exits_columns} for item in self.data]

    def to_dict(
            self,
            chinese_column: Optional[bool] = False,
    ) -> List[Dict]:
        """
        Args:
            chinese_column: 是否将列名翻译为中文

        Returns:
            List[Dict]: 转换后的数据
        """
        if chinese_column:
            data = [self._trans_chinese_columns(item) for item in self.data]
        else:
            data = self.data
        return data

    def to_frame(
            self,
            chinese_column: Optional[bool] = False,
    ) -> pd.DataFrame:
        """返回数据的 pandas.DataFrame

        Args:
            chinese_column: 是否将列名翻译为中文

        Returns:
            dataframe: 数据的 pandas.DataFrame
        """
        df = pd.DataFrame(data=self.data)
        if chinese_column:
            df.rename(columns=self.metadata.columns, inplace=True)
        return df

    def to_markdown(
            self,
            chinese_column: Optional[bool] = False,
            **kwargs: Any,
    ) -> str:
        """返回 Markdown 格式的数据

        Args:
            chinese_column: 是否将列名翻译为中文
            **kwargs: to_markdown() 的参数

        Returns:
            str: markdown 格式的数据
        """
        return self.to_frame(chinese_column).to_markdown(**kwargs)

    def to_csv(
            self,
            path: str,
            chinese_column: Optional[bool] = False,
            **kwargs: Any
    ) -> None:
        """将数据保存至 csv 文件中

        Args:
            path: csv 文件路径
            chinese_column: 是否将列名翻译为中文
            **kwargs: 其他参数
        """
        self.to_frame(chinese_column).to_csv(path, **kwargs)

    def to_excel(
            self,
            path: str,
            chinese_column: Optional[bool] = False,
            **kwargs: Any
    ) -> None:
        """将数据保存至 excel 文件中

        Args:
            path: excel 文件路径
            chinese_column: 是否将列名翻译为中文
            **kwargs: 其他参数
        """
        self.to_frame(chinese_column).to_excel(path, **kwargs)

    def to_parquet(
            self,
            path: str,
            chinese_column: Optional[bool] = False,
            index: Optional[bool] = False,
            engine: Literal["auto", "pyarrow", "fastparquet"] = "auto",
            compression: str | None = "snappy",
            **kwargs: Any
    ) -> None:
        """将数据存储为 parquet 文件

        Args:
            path: 文件路径
            chinese_column: 是否使用中文列名
            index: 是否包含索引
            engine: 存储引擎
            compression: 压缩格式
            **kwargs: 其他参数
        """
        self.to_frame(chinese_column).to_parquet(
            path, index=index, engine=engine,
            compression=compression, **kwargs)

    def to_pickle(
            self,
            path: str,
            chinese_column: Optional[bool] = False,
            **kwargs: Any,
    ):
        """将数据存储为 pickle 文件

        Args:
            path: 文件保存路径
            chinese_column: 是否使用中文列名
            **kwargs: 其他参数
        """
        self.to_frame(chinese_column).to_pickle(path,  **kwargs)

    def to_string(
            self,
            chinese_column: Optional[bool] = False,
            **kwargs: Any
    ) -> str:
        """将数据转换为 string 格式

        Args:
            chinese_column: 是否将列名转换为中文
            **kwargs: 其他参数

        Returns:
            str: 数据的 string 格式
        """
        return self.to_frame(chinese_column).to_string(**kwargs)

    def to_sql(
            self,
            name: str,
            con: Any,
            chinese_column: Optional[bool] = False,
            if_exists: Literal["fail", "replace", "append"] = "fail",
            index: Optional[bool] = True,
            **kwargs: Any,
    ):
        """将数据存储至 Database 中

        Args:
            name: 表名
            con: 数据库连接
            chinese_column: 是否将列名转为中文
            if_exists: 如果表已存在，则采取的操作
            index: 是否将索引存储至数据库中
            **kwargs: 其他参数
        """
        self.to_frame(chinese_column).to_sql(
            name=name, con=con, if_exists=if_exists,
            index=index, **kwargs)

    def to_duckdb(
            self,
            database: str,
            name: str,
            chinese_column: Optional[bool] = False,
            if_exists: Literal["fail", "replace", "append"] = "replace",
            **kwargs: Any,
    ):
        """将数据存储至 duckdb 数据库中

        Args:
            database: 数据库文件路径
            name: 表名
            chinese_column: 是否将列名转为中文
            if_exists: 如果表存在，是否覆盖
            **kwargs: 其他参数
        """
        from sqlalchemy import create_engine

        df = self.to_frame(chinese_column)
        engine = create_engine(database, **kwargs)
        with engine.connect() as con:
            df.to_sql(name, con=con, if_exists=if_exists, index=False)



import json
from typing import Any, List, Dict, Callable, Literal, Optional
from wisecon.types import BaseMapping, BaseRequestData, Metadata
from wisecon.utils import headers, jquery_mock_callback, time2int, filter_str_by_mark


__all__ = [
    "FundTrendMapping",
    "FundTrend",
]


class FundTrendMapping(BaseMapping):
    """"""
    columns: Dict = {
        "date": "日期",
        "value": "累计收益",
        "hs_300": "沪深300",
        "sh_index": "上证指数",
    }


class FundTrend(BaseRequestData):
    """ Fund Trend """
    def __init__(
            self,
            fund_code: str,
            period: Literal["1m", "3m", "6m", "1y", "3y", "5y", "this_year", "all"] = "1y",
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """"""
        self.fund_code = fund_code
        self.period = period
        self.mapping = FundTrendMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

        self.request_set(
            response_type="text",
            description="基金趋势",
            other_headers={'Referer': 'https://fundf10.eastmoney.com/'},
        )

    def base_url(self) -> str:
        """"""
        base_url = "https://api.fund.eastmoney.com/f10/FundLJSYLZS/"
        return base_url

    def _param_dt(self) -> str:
        """"""
        dt_mapping = {
            "1m": "month", "3m": "threemonth", "6m": "sixmonth",
            "1y": "year", "3y": "threeyear", "5y": "fiveyear",
            "this_year": "thisyear", "all": "all"}
        return dt_mapping[self.period]

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "callback": jquery_mock_callback(),
            "bzdm": self.fund_code,
            "dt": self._param_dt(),
            "_": time2int(),
        }
        return params

    def clean_content(
            self,
            content: Optional[str]
    ) -> List[Dict]:
        """"""
        content = filter_str_by_mark(content)
        response = json.loads(content)
        origin_data = response.pop("Data")
        columns = list(self.mapping.columns)
        data = [dict(zip(columns, item.split("_"))) for item in origin_data.split("|")]
        self.metadata.response = response
        return data



import pprint
import unittest
from wisecon.stock.capital_flow import *


class TestPlateFlow(unittest.TestCase):

    def test_columns(self):
        """"""
        data = PlateFlow(days=1, size=10).load()
        pprint.pprint(data.to_dict(chinese_column=True)[0])
        print(data.metadata)

    def test_moke(self):
        """"""
        data = PlateFlow(days=1).load()
        data.show_columns()
        print(data.to_frame().to_markdown())
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_market(self):
        """"""
        data = PlateFlow(market="沪深A股", days=1).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_type_indu(self):
        """"""
        data = PlateFlow(plate_type="行业", days=1).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_type_zone(self):
        """"""
        data = PlateFlow(plate_type="地区", days=1).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_type_conception(self):
        """"""
        data = PlateFlow(plate_type="概念", days=1).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_code_industry(self):
        """"""
        data = PlateFlow(plate_code="BK1027", days=1).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_code_zone(self):
        """"""
        data = PlateFlow(plate_code="BK0158", days=1).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_code_conception(self):
        """"""
        data = PlateFlow(plate_code="BK1044", days=1).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_sort(self):
        """"""
        data = PlateFlow(market="全部股票", sort_by="f184").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_type_indu_sort(self):
        """"""
        data = PlateFlow(plate_type="行业", days=1, ascending=False).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_type_indu_sort_true(self):
        """"""
        data = PlateFlow(plate_type="行业", days=1, ascending=True).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())


class TestHistoryCapitalFlow(unittest.TestCase):

    def test_columns(self):
        """"""
        data = CapitalFlowHistory(market="沪B", verbose=True, size=10).load()
        pprint.pprint(data.to_dict(chinese_column=True)[-1])
        print(data.metadata)

    def test_market_history(self):
        """"""
        data = CapitalFlowHistory(market="沪深两市", verbose=True, size=10).load()
        print(data.to_frame().to_markdown())
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_history_conception(self):
        """"""
        data = CapitalFlowHistory(plate_code="BK1044", verbose=True, size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_history_zone(self):
        """BK0158"""
        data = CapitalFlowHistory(plate_code="BK0158", verbose=True, size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_history_industry(self):
        """BK1027"""
        data = CapitalFlowHistory(plate_code="BK1027", verbose=True, size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_security_code(self):
        """"""
        data = CapitalFlowHistory(security_code="300750", size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())


class TestCurrentCapitalFlow(unittest.TestCase):
    """"""

    def test_market(self):
        """"""
        data = CapitalFlowCurrent(market="沪深两市", verbose=True, size=10).load()
        pprint.pprint(data.to_dict(chinese_column=True)[-1])
        print(data.metadata)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_market_sh(self):
        """"""
        data = CapitalFlowCurrent(market="沪市", verbose=True, size=10).load()
        print(data.to_markdown(chinese_column=True))

    def test_stock(self):
        """"""
        data = CapitalFlowCurrent(security_code=["601012", "300750"], verbose=True, size=10).load()
        pprint.pprint(data.to_dict(chinese_column=True)[-1])
        print(data.metadata)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate(self):
        """"""
        data = CapitalFlowCurrent(plate_code=["BK1044", "BK1027", "BK0158"], verbose=True, size=10).load()
        pprint.pprint(data.to_dict(chinese_column=True)[-1])
        print(data.metadata)
        print(data.to_frame(chinese_column=True).to_markdown())


class TestMinutesCapitalFlow(unittest.TestCase):

    def test_columns(self):
        """"""
        data = CapitalFlowMinutes(market="沪B", verbose=True, size=10).load()
        pprint.pprint(data.to_dict(chinese_column=True)[-1])
        print(data.metadata)

    def test_market_minutes(self):
        """"""
        data = CapitalFlowMinutes(market="沪深两市", verbose=True, size=10).load()
        print(data.to_frame().to_markdown())
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_market_minutes_sh(self):
        """"""
        data = CapitalFlowMinutes(market="沪市", verbose=True, size=10).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_minutes_conception(self):
        """"""
        data = CapitalFlowMinutes(plate_code="BK1044", verbose=True, size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_Minutes_zone(self):
        """BK0158"""
        data = CapitalFlowMinutes(plate_code="BK0158", verbose=True, size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_Minutes_industry(self):
        """BK1027"""
        data = CapitalFlowMinutes(plate_code="BK1027", verbose=True, size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_security_code(self):
        """"""
        data = CapitalFlowMinutes(security_code="300750", size=10).load()
        print(data.metadata.response)
        print(data.to_frame(chinese_column=True).to_markdown())



import unittest
from wisecon.car import *


class TestCarBrand(unittest.TestCase):
    """"""
    def test_00(self):
        data = CarBrandList(verbose=True).load()
        print(data.to_frame(chinese_column=True))


class TestCarCurrent(unittest.TestCase):
    """"""
    def test_01(self):
        data = CurrentCarSales(data_type="销量", verbose=True).load()
        print(data.to_frame(chinese_column=True))

    def test_02(self):
        data = CurrentCarSales(data_type="车型级别", verbose=True).load()
        print(data.to_frame(chinese_column=True))

    def test_03(self):
        data = CurrentCarSales(data_type="厂商销量", verbose=True).load()
        print(data.to_frame(chinese_column=True))

    def test_04(self):
        data = CurrentCarSales(data_type="车型销量", verbose=True).load()
        print(data.to_frame(chinese_column=True))

    def test_05(self):
        data = CurrentCarSales(data_type="车身类别", verbose=True).load()
        print(data.to_frame(chinese_column=True))

    def test_06(self):
        data = CurrentCarSales(data_type="品牌销量", verbose=True).load()
        print(data.to_frame(chinese_column=True))

    def test_07(self):
        data = CurrentCarSales(data_type="电动车销量", verbose=True).load()
        print(data.to_frame(chinese_column=True))


class TestHistory(unittest.TestCase):
    """"""
    def test_html(self):
        """"""
        data = CarHistorySales(data_type="汽车销量").load()
        df = data.to_frame(chinese_column=True)
        print(df.shape)
        print(df.to_markdown())



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "Earn",
    "EarnMapping",
]


class EarnMapping(BaseMapping):
    """字段映射 上市公司业绩报表"""
    columns: Dict = {
        "SECURITY_CODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "TRADE_MARKET_CODE": "交易市场代码",
        "TRADE_MARKET": "交易市场",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "SECURITY_TYPE": "证券类型",
        "UPDATE_DATE": "更新时间",
        "REPORTDATE": "报告日期",
        "BASIC_EPS": "每股收益(元)",
        "DEDUCT_BASIC_EPS": "扣除后基本每股收益",
        "TOTAL_OPERATE_INCOME": "营业总收入",
        "PARENT_NETPROFIT": "净利润",
        "WEIGHTAVG_ROE": "净资产收益率",
        "YSTZ": "营业收入同比增长(%)",
        "SJLTZ": "净利润同比增长(%)",
        "BPS": "每股净资产(元)",
        "MGJYXJJE": "每股经营现金流量(元)",
        "XSMLL": "销售毛利率",
        "YSHZ": "营业收入季度环比增长率(%)",
        "SJLHZ": "净利润季度环比增长率(%)",
        "ASSIGNDSCRPT": "分配说明",
        "PAYYEAR": "支付年份",
        "PUBLISHNAME": "行业",
        "ZXGXL": "资产结构",
        "NOTICE_DATE": "公告日期",
        "ORG_CODE": "机构代码",
        "TRADE_MARKET_ZJG": "交易市场信息",
        "ISNEW": "是否最新",
        "QDATE": "季度日期",
        "DATATYPE": "数据类型",
        "DATAYEAR": "数据年份",
        "DATEMMDD": "数据日期",
        "EITIME": "导入时间",
        "SECUCODE": "证券代码"
    }


class Earn(APIDataV1RequestData):
    """查询 上市公司业绩报表"""
    def __init__(
            self,
            security_code: Optional[str] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            industry_name: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.financial import Earn

            data = Earn(date="2024-09-30", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            industry_name: 行业名称
            size: 数据条数据
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.industry_name = industry_name
        self.mapping = EarnMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司业绩报表")
        self.conditions = []

    def params_filter(self) -> str:
        """
        :return: 
        """
        if self.start_date:
            self.conditions.append(f"(REPORTDATE>='{self.start_date}')")
        if self.end_date:
            self.conditions.append(f"(REPORTDATE<='{self.end_date}')")
        if self.date:
            self.conditions.append(f"(REPORTDATE='{self.date}')")

        if self.security_code:
            self.conditions.append(f'(SECURITY_CODE="{self.security_code}")')

        if self.industry_name:
            self.conditions.append(f'(PUBLISHNAME="{self.industry_name}")')

        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "UPDATE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_LICO_FN_CPD",
        }
        return self.base_param(params)



import os
from logging import Logger
from openai import OpenAI
from zhipuai import ZhipuAI
from pydantic import BaseModel, ConfigDict
from typing import Any, List, Callable, Optional, Literal, Union
from openai._streaming import Stream
from openai.types.chat.chat_completion import (
    ChatCompletion)
from openai.types.chat.chat_completion_chunk import (
    ChatCompletionChunk, Choice, ChoiceDelta)

from wisecon.types.message import Message, ToolMessage
from wisecon.agent.tools.tools import Tools
from wisecon.utils.logger import LoggerMixin


__all__ = ["BaseAgent"]


class BaseAgent(BaseModel, LoggerMixin):
    """
    todo: 1. 区分Agent模型与Reasoning模型
    todo: 2. 增加sse模式
    """
    model_config = ConfigDict(arbitrary_types_allowed=True)
    client: Union[OpenAI, ZhipuAI, Literal["openai", "zhipu"]]
    model: str
    api_key: Optional[str]
    api_key_name: Optional[str]
    base_url: str
    tools: Tools

    def __init__(
            self,
            model: str,
            api_key: Optional[str] = None,
            base_url: Optional[str] = "https://api.openai.com/v1",
            api_key_name: Optional[str] = None,
            tools: List[Any] = None,
            client: Union[OpenAI, ZhipuAI, Literal["openai", "zhipu"]] = "openai",
            logger: Optional[Union[Logger, Callable]] = None,
            verbose: Optional[bool] = True,
            **kwargs
    ):
        """"""
        super().__init__(
            api_key=api_key, base_url=base_url, model=model,
            tools=tools, api_key_name=api_key_name,
            client=client, logger=logger, verbose=verbose,
            **kwargs)
        self.set_api_key(api_key, api_key_name)
        self.set_client(client)
        self.logger = logger
        self.verbose = verbose

    def set_client(
            self,
            client: Union[OpenAI, ZhipuAI, Literal["openai", "zhipu"]] = "openai",
    ):
        """"""
        if isinstance(client, (OpenAI, ZhipuAI)):
            self.client = client
        elif client == "zhipu":
            self.client = ZhipuAI(api_key=self.api_key, base_url=self.base_url)
        elif client == "openai":
            self.client = OpenAI(api_key=self.api_key, base_url=self.base_url)
        else:
            raise ValueError(f"Client `{client}` is not supported.")

    def set_api_key(self, api_key: Optional[str] = None, api_key_name: Optional[str] = None):
        """"""
        if api_key is not None:
            self.api_key = api_key
        elif api_key_name is not None:
            self.api_key = os.getenv(self.api_key_name)
        if self.api_key is None:
            raise ValueError("API key not found")

    def _chunk(
            self,
            role: Optional[Literal["developer", "system", "user", "assistant", "tool"]],
            content: str,
            chunk: ChatCompletionChunk,
    ) -> ChatCompletionChunk:
        """"""
        choices = [Choice(delta=ChoiceDelta(role=role, content=content), index=0)]
        return ChatCompletionChunk(
            choices=choices, id=chunk.id, created=chunk.created, model=chunk.model,
            object="chat.completion.chunk")

    def chat(
            self,
            prompt: Optional[str] = None,
            messages: Optional[List[Message]] = None,
            stream: Optional[bool] = False
    ) -> Union[ChatCompletion | Stream[ChatCompletionChunk]]:
        """"""
        if stream:
            return self.sse(prompt, messages)
        else:
            return self.sync(prompt, messages)

    def sync(
            self,
            prompt: Optional[str] = None,
            messages: Optional[List[Message]] = None,
    ) -> ChatCompletion:
        """"""
        if prompt is not None:
            messages = [Message(role="user", content=prompt)]
        self._logger(msg=f"[User] {messages[-1].content}\n", color="blue")

        completion = self.client.chat.completions.create(
            model=self.model, messages=[msg.to_dict() for msg in messages], tools=self.tools.descriptions)

        if completion.choices[0].finish_reason == "tool_calls":
            messages.append(completion.choices[0].message)
            function = completion.choices[0].message.tool_calls[0].function
            self._logger(msg=f"[Assistant] name: {function.name}, arguments: {function.arguments}\n", color="green")

            observation = self.tools.dispatch(function=function)
            self._logger(msg=f"[Observation] \n{observation}\n", color="magenta")

            messages.append(ToolMessage(role="tool", content=observation, tool_call_id=completion.choices[0].message.tool_calls[0].id))
            completion = self.client.chat.completions.create(
                model=self.model, messages=[msg.to_dict() for msg in messages], tools=self.tools.descriptions)
        content = completion.choices[0].message.content
        self._logger(msg=f"[Assistant] {content}\n", color="green")
        return completion

    def sse(
            self,
            prompt: Optional[str] = None,
            messages: Optional[List[Message]] = None,
    ) -> Stream[ChatCompletionChunk]:
        """"""
        if prompt is not None:
            messages = [Message(role="user", content=prompt)]

        completion = self.client.chat.completions.create(
            model=self.model, messages=[msg.to_dict() for msg in messages], tools=self.tools.descriptions, stream=True)

        chunk = None
        function = None
        tool_call_id = None
        for chunk in completion:
            if chunk.choices[0].delta.tool_calls:
                function = chunk.choices[0].delta.tool_calls[0].function
                tool_call_id = chunk.choices[0].delta.tool_calls[0].id
                messages.append(chunk.choices[0].delta)
            yield chunk

        if function is not None:
            yield self._chunk(role="assistant", content="\n\n", chunk=chunk)
            observation = self.tools.dispatch(function=function)
            yield self._chunk(role="tool", content=observation, chunk=chunk)
            yield self._chunk(role="tool", content="\n\n", chunk=chunk)
            messages.append(
                ToolMessage(role="tool", content=observation, tool_call_id=tool_call_id))
            completion = self.client.chat.completions.create(
                model=self.model, messages=[msg.to_dict() for msg in messages],
                tools=self.tools.descriptions, stream=True)
            for chunk in completion:
                yield chunk



from .stock_rank_detail import *
from .stock_rank_statistic import *
from .institution_trade import *
from .institution_seat import *
from .department_active import *
from .department_rank import *
from .department_statistic import *



import time
import requests
from pydantic import BaseModel
from typing import Dict, Optional
from wisecon.utils import headers
from wisecon.utils import LoggerMixin


__all__ = [
    "assemble_url",
    "BaseRequestConfig",
    "BaseRequestData",
]


def assemble_url(base_url: str, params: Dict) -> str:
    """"""
    query_string = '&'.join([f"{key}={value}" for key, value in params.items()])
    request_url = f"{base_url}?{query_string}"
    return request_url


class BaseRequestConfig(BaseModel):
    """"""
    def _current_time(self) -> str:
        """"""
        return str(int(time.time() * 1E3))

    def params(self) -> Dict:
        """"""
        return dict()


class BaseRequestData(LoggerMixin):
    """"""
    query_config: Optional[BaseRequestConfig]

    def _base_url(self) -> str:
        """"""
        return ""

    def request_json(self) -> Dict:
        """"""
        base_url = self._base_url()
        params = self.query_config.to_params()
        response = requests.get(base_url, params=params, headers=headers)
        data = response.json()
        return data



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "GoodsIndexMapping",
    "GoodsIndex",
]


class GoodsIndexMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "BASE": "总指数",
        "BASE_SAME": "总指数（同比）",
        "BASE_SEQUENTIAL": "总指数（环比）",
        "FARM_BASE": "农产品",
        "FARM_BASE_SAME": "农产品（同比）",
        "FARM_BASE_SEQUENTIAL": "农产品（环比）",
        "MINERAL_BASE": "矿产品",
        "MINERAL_BASE_SAME": "矿产品（同比）",
        "MINERAL_BASE_SEQUENTIAL": "矿产品（环比）",
        "ENERGY_BASE": "煤油电",
        "ENERGY_BASE_SAME": "煤油电（同比）",
        "ENERGY_BASE_SEQUENTIAL": "煤油电（环比）",
    }


class GoodsIndex(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = GoodsIndexMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 企业商品价格指数")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "BASE", "BASE_SAME", "BASE_SEQUENTIAL", "FARM_BASE", "FARM_BASE_SAME",
            "FARM_BASE_SEQUENTIAL", "MINERAL_BASE", "MINERAL_BASE_SAME", "MINERAL_BASE_SEQUENTIAL", "ENERGY_BASE",
            "ENERGY_BASE_SAME", "ENERGY_BASE_SEQUENTIAL"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_GOODS_INDEX",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "IndexStock",
    "IndexStockMapping",
]


class IndexStockMapping(BaseMapping):
    """字段映射 指数成分股数据"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "TYPE": "类型",
        "SECURITY_NAME_ABBR": "证券简称",
        "CLOSE_PRICE": "收盘价",
        "INDUSTRY": "行业",
        "REGION": "地区",
        "WEIGHT": "权重",
        "EPS": "每股收益",
        "BPS": "每股净资产",
        "ROE": "净资产收益率",
        "TOTAL_SHARES": "总股份",
        "FREE_SHARES": "自由流通股份",
        "FREE_CAP": "自由流通市值",
        "f2": "最新价",
        "f3": "涨跌幅"
    }


class IndexStock(APIDataV1RequestData):
    """查询 指数成分股数据"""
    def __init__(
            self,
            index_name: Optional[Literal["沪深300", "上证50", "中证500", "科创50"]] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.index import *

            data = IndexStock(index_name="沪深300", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            index_name: 指数名称`["沪深300", "上证50", "中证500", "科创50"]`
            size: 返回条数
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.index_name = index_name
        self.size = size
        self.mapping = IndexStockMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="指数成分股数据")
        self.conditions = []

    def params_index_name(self) -> str:
        """"""
        index_name_mapping = {"沪深300": "1", "上证50": "2", "中证500": "3", "科创50": "4"}
        if self.index_name:
            return index_name_mapping.get(self.index_name, "1")
        else:
            return "1"

    def params_filter(self) -> str:
        """"""
        self.conditions.append(f'(TYPE="{self.params_index_name()}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "SECUCODE", "SECURITY_CODE", "TYPE", "SECURITY_NAME_ABBR", "CLOSE_PRICE",
            "INDUSTRY", "REGION", "WEIGHT", "EPS", "BPS", "ROE", "TOTAL_SHARES",
            "FREE_SHARES", "FREE_CAP",
        ]

        params = {
            "columns": ",".join(columns),
            "quoteColumns": "f2,f3",
            "quoteType": "0",
            "filter": self.params_filter(),
            "sortColumns": "SECURITY_CODE",
            "sortTypes": "-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_INDEX_TS_COMPONENT",
        }
        return self.base_param(params)



import unittest
from wisecon.movie import *
from wisecon.agent import *


class TestMovie(unittest.TestCase):
    """"""

    def test_tv_0(self):
        """"""
        data = TV(source="央视").load()
        print(data.to_markdown(chinese_column=True))

    def test_tv_1(self):
        """"""
        data = TV(source="卫视").load()
        print(data.to_markdown(chinese_column=True))

    def test_movie(self):
        """"""
        data = Movie().load()
        print(data.to_markdown(chinese_column=True))

    def test_movie_params(self):
        """"""
        data = Movie(verbose=True).load()
        print(data.to_markdown(chinese_column=True))

    def test_tools(self):
        """"""
        base_url = "https://open.bigmodel.cn/api/paas/v4"
        tools = Tools(tools=[Movie, TV])
        agent = BaseAgent(model="glm-4-flash", base_url=base_url, tools=tools, api_key_name="ZHIPU_API_KEY")
        completion = agent.chat(prompt="现在央视收视率最高的节目是？")



from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "MarginTradingAccountMapping",
    "MarginTradingAccount",
]


class MarginTradingAccountMapping(BaseMapping):
    """字段映射 融资融券账户统计-两融账户信息"""
    columns: Dict = {
        "STATISTICS_DATE": "统计日期",
        "DATE_MARK": "日期标识",
        "TARGET_SECURITY_NUM": "目标证券数量",
        "FIN_BUY_AMT": "融资买入金额",
        "LOAN_SELL_AMT": "借入卖出金额",
        "MARGIN_TRADE_AMT": "保证金交易金额",
        "FIN_BALANCE": "融资余额",
        "LOAN_BALANCE": "借入余额",
        "MARGIN_BALANCE": "保证金余额",
        "SECURITY_ORG_NUM": "证券机构数量",
        "OPERATEDEPT_NUM": "操作部门数量",
        "END_ACC_NUM": "账户总数",
        "ADD_ACC_NUM": "新增账户数量",
        "FUND_MARKET_CAP": "基金市场总值",
        "STOCK_MARKET_CAP": "股票市场总值",
        "OTHER_MARKET_CAP": "其他市场总值",
        "TOTAL_MARKET_CAP": "市场总值",
        "TOTAL_GUARANTEE": "总保证金",
        "AVG_GUARANTEE_RATIO": "平均保证金比例",
        "SCI_CLOSE_PRICE": "SCI收盘价",
        "SCI_CHANGE_RATE": "SCI变动率",
        "ADD_ACC_RATIO": "新增账户比例",
        "BALANCE_RATIO": "余额比例",
        "TRADE_AMT_RATIO": "交易金额比例",
        "PERSONAL_INVESTOR_NUM": "个人投资者数量",
        "ORG_INVESTOR_NUM": "机构投资者数量",
        "INVESTOR_NUM": "投资者总数",
        "MARGINLIAB_INVESTOR_NUM": "保证金负债投资者数量",
        "SUBSITUTION__CAP": "替代资本"
    }


class MarginTradingAccount(APIDataV1RequestData):
    """查询 融资融券账户统计-两融账户信息"""
    def __init__(
            self,
            cycle: Literal["day", "month"] = "day",
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            size: Optional[int] = 100,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.margin import *

            # 查询 融资融券账户统计-两融账户信息
            data = MarginTradingAccount(market="全部").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            cycle: 统计周期
            start_date: 开始日期
            end_date: 结束日期
            date: 日期
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.cycle = cycle
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.size = size
        self.mapping = MarginTradingAccountMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="融资融券账户统计-两融账户信息")
        self.conditions = []
        self.validate_date_format(date=[date, start_date, end_date])

    def params_report_name(self) -> str:
        """"""
        if self.cycle == "day":
            return "RPTA_WEB_MARGIN_DAILYTRADE"
        else:
            return "RPTA_WEB_MARGIN_MONTHTRADE"

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="STATISTICS_DATE")
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        params = {
            "reportName": self.params_report_name(),
            "sortColumns": "STATISTICS_DATE",
            "sortTypes": "-1",
            "pageSize": self.size,
            "filter": self.params_filter(),
            "pageNo": "1",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "LPRMapping",
    "LPR",
]


class LPRMapping(BaseMapping):
    """"""
    columns: Dict = {
        "TRADE_DATE": "日期",
        "LPR1Y": "LPR_1Y利率(%)",
        "LPR5Y": "LPR_5Y利率(%)",
        "RATE_1": "短期贷款利率:6个月至1年(含)(%)",
        "RATE_2": "中长期贷款利率:5年以上(%)",
    }


class LPR(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = LPRMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国 LPR品种详细数据",
        )

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "TRADE_DATE", "LPR1Y", "LPR5Y", "RATE_1", "RATE_2"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "TRADE_DATE",
            "sortTypes": "-1",
            "reportName": "RPTA_WEB_RATE",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional, Literal, Annotated
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "ETFGoldHistoryMapping",
    "ETFGoldHistory",
]


class ETFGoldHistoryMapping(BaseMapping):
    """字段映射 白银/黄金ETF历史行情"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "INDICATOR_NAME": "指标名称",
        "INDICATOR_ID1": "指标ID1",
        "STORAGE_TON": "库存（吨）",
        "STORAGE_OUNCE": "库存（盎司）",
        "INDICATOR_ID2": "指标ID2",
        "NETPOSITION_TON": "净头寸（吨）",
        "NETPOSITION_OUNCE": "净头寸（盎司）",
        "NETPOSITION_DOLLAR": "净头寸（美元）",
        "INDICATOR_ID3": "指标ID3",
        "OPENPOSI_STOCK": "开盘持仓",
        "OPENPOSTOCK_WECHANGE": "开盘持仓变动",
        "OPENPOSI_STOCK_SUM": "开盘持仓总和"
    }


class ETFGoldHistory(APIDataV1RequestData):
    """查询 白银/黄金ETF历史行情"""
    def __init__(
            self,
            market: Annotated[Literal["ETF白银", "ETF黄金"], "只能指定['ETF白银', 'ETF黄金']两个市场", False] = "ETF黄金",
            start_date: Annotated[Optional[str], "", False] = None,
            end_date: Annotated[Optional[str], "", False] = None,
            date: Annotated[Optional[str], "", False] = None,
            size: Annotated[Optional[int], "", False] = 100,
            verbose: Annotated[Optional[bool], "", False] = False,
            logger: Annotated[Optional[Callable], "", False] = None,
            **kwargs: Annotated[Any, "", False]
    ):
        """
        Notes:
            ```python
            from wisecon.stock.etf import *

            # 1. 查询 ETF白银历史行情
            data = ETFGoldHistory(market="ETF白银").load()
            data.to_frame(chinese_column=True)

            # 2. 查询 ETF黄金某时点
            data = ETFGoldHistory(market="ETF黄金", date="2024-10-25").load()
            data.to_frame(chinese_column=True)

            # 3. 查询 ETF黄金历史行情，并指定查询开始日期
            data = ETFGoldHistory(market="ETF黄金", start_date="2024-10-25").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场类型
            start_date: 开始日期
            end_date: 结束日期
            date: 查询日期
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.market = market
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = ETFGoldHistoryMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="白银/黄金ETF历史行情")
        self.conditions = []
        self.validate_date_format(date=[start_date, end_date, date])

    def params_filter(self) -> str:
        """"""
        market_mapping = {
            "ETF白银": '(INDICATOR_ID2="EMI00223868")(@NETPOSITION_TON<>"NULL")',
            "ETF黄金": '(INDICATOR_ID2="EMI00223865")(@NETPOSITION_TON<>"NULL")',
        }
        self.conditions.append(market_mapping[self.market])
        self.filter_report_date(date_name="REPORT_DATE")
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        params = {
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "pageSize": self.size,
            "reportName": "RPT_FUTUOPT_GOLDSIL",
            "filter": self.params_filter(),
        }
        return self.base_param(update=params)



from typing import List, Union, Literal, Optional
from pandas import DataFrame


__all__ = [
    "transform_tool_params",
]





def transform_tool_params(data):
    """Transform the data structure to the desired format."""
    if isinstance(data, dict) and "Items" in data:
        return data["Items"]
    elif isinstance(data, dict):
        return {k: transform_tool_params(v) for k, v in data.items()}
    else:
        return data




from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "DepositReserveMapping",
    "DepositReserve",
]


class DepositReserveMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告时间",
        "PUBLISH_DATE": "公布时间",
        "TRADE_DATE": "生效时间",
        "INTEREST_RATE_BB": "大型金融机构（调整前）",
        "INTEREST_RATE_BA": "大型金融机构（调整后）",
        "CHANGE_RATE_B": "大型金融机构（调整幅度）",
        "INTEREST_RATE_SB": "中小金融机构（调整前）",
        "INTEREST_RATE_SA": "中小金融机构（调整后）",
        "CHANGE_RATE_S": "中小金融机构（调整幅度）",
        "NEXT_SH_RATE": "消息公布次日指数涨跌（上证-SH）",
        "NEXT_SZ_RATE": "消息公布次日指数涨跌（深证-SZ）",
        "REMARK": "消息",
    }


class DepositReserve(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = DepositReserveMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 存款准备金率")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "PUBLISH_DATE", "TRADE_DATE", "INTEREST_RATE_BB", "INTEREST_RATE_BA",
            "CHANGE_RATE_B", "INTEREST_RATE_SB", "INTEREST_RATE_SA", "CHANGE_RATE_S", "NEXT_SH_RATE",
            "NEXT_SZ_RATE", "REMARK",
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "PUBLISH_DATE,TRADE_DATE",
            "sortTypes": "-1,-1",
            "reportName": "RPT_ECONOMY_DEPOSIT_RESERVE",
        }
        return self.base_param(update=params)



import unittest
from pprint import pprint
from wisecon.stock.holder import *


class TestFreeHolder(unittest.TestCase):

    def test_columns(self):
        """"""
        data = FreeHolder(start_date="2024-09-30", size=1).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = FreeHolder(start_date="2024-09-30", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = FreeHolder(start_date="2024-06-30", security_code="000766", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestHolder(unittest.TestCase):

    def test_columns(self):
        """"""
        data = Holder(start_date="2024-09-30", size=4).load()
        pprint(data.to_dict(chinese_column=True)[3], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = Holder(start_date="2024-09-30", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = Holder(start_date="2024-06-30", security_code="000766", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestFreeHolderChange(unittest.TestCase):

    def test_columns(self):
        """"""
        data = FreeHolderChange(start_date="2015-03-31", size=50, verbose=True).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = FreeHolderChange(start_date="2015-03-31", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = FreeHolderChange(start_date="2015-03-31", holder_name="工商银行", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestHolderChange(unittest.TestCase):

    def test_columns(self):
        """"""
        data = HolderChange(start_date="2015-03-31", size=50, verbose=True).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = HolderChange(start_date="2015-03-31", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = HolderChange(start_date="2015-03-31", holder_name="工商银行", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestFreeHolderStock(unittest.TestCase):

    def test_columns(self):
        """"""
        data = FreeHolderStock(start_date="2024-09-30", size=5, verbose=True).load()
        pprint(data.to_dict(chinese_column=True)[3], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = FreeHolderStock(start_date="2024-09-30", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = FreeHolderStock(start_date="2024-09-30", holder_name="工商银行", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestHolderStock(unittest.TestCase):

    def test_columns(self):
        """"""
        data = HolderStock(start_date="2024-09-30", size=5, verbose=True).load()
        pprint(data.to_dict(chinese_column=True)[3], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = HolderStock(start_date="2024-09-30", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = HolderStock(start_date="2024-09-30", holder_name="工商银行", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestFreeHolderStockAnalysis(unittest.TestCase):

    def test_columns(self):
        """"""
        data = FreeHolderStockAnalysis(start_date="2015-03-31", size=5).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = FreeHolderStockAnalysis(start_date="2015-03-31", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = FreeHolderStockAnalysis(start_date="2015-03-31", holder_name="工商银行", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestHolderStockAnalysis(unittest.TestCase):

    def test_columns(self):
        """"""
        data = HolderStockAnalysis(start_date="2015-03-31", size=5).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = HolderStockAnalysis(start_date="2015-03-31", size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = HolderStockAnalysis(start_date="2015-03-31", holder_name="工商银行", size=10, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestFreeHolderCoop(unittest.TestCase):

    def test_columns(self):
        """"""
        data = FreeHolderCoop(size=5).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = FreeHolderCoop(size=5).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = FreeHolderCoop(holder_name="工商银行", size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestHolderCoop(unittest.TestCase):

    def test_columns(self):
        """"""
        data = HolderCoop(size=5).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = HolderCoop(size=5).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        data = HolderCoop(holder_name="工商银行", size=10).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())



from typing import Any, Dict, Literal, Callable, Optional
from .base import CapitalFlowHistoryBaseMapping, CapitalFlowHistoryRequestData

__all__ = [
    "CapitalFlowHistoryMapping",
    "CapitalFlowHistory",
]


TypeMarket = Literal["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]


class CapitalFlowHistoryMapping(CapitalFlowHistoryBaseMapping):
    """字段映射 资金流向历史数据(大盘沪深两市/板块历史数据)"""


class CapitalFlowHistory(CapitalFlowHistoryRequestData):
    """查询 资金流向历史数据(大盘沪深两市/板块历史数据)

    1. 查询沪深两市市场历史资金动向数据
    2. 查询某行业、地区、概念板块历史资金动向数据
    3. 查询具体股票的历史资金动向数据
    """
    def __init__(
            self,
            market: Optional[TypeMarket] = None,
            plate_code: Optional[str] = None,
            security_code: Optional[str] = None,
            size: Optional[int] = 0,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.capital_flow import *

            # 1. 查询市场的资金历史流向数据
            data = CapitalFlowHistory(market="沪深两市", size=10).load()
            data.to_frame(chinese_column=True)

            # 2.1 查询板块的资金历史流向数据(概念)
            data = CapitalFlowHistory(plate_code="BK1044", size=10).load()
            data.to_frame(chinese_column=True)

            # 2.2 查询板块的资金历史流向数据(地区)
            data = CapitalFlowHistory(plate_code="BK0158", size=10).load()
            data.to_frame(chinese_column=True)

            # 2.3 查询板块的资金历史流向数据(行业)
            data = CapitalFlowHistory(plate_code="BK1044", size=10).load()
            data.to_frame(chinese_column=True)

            # 3. 查询股票的资金历史流向数据
            data = CapitalFlowHistory(security_code="300750", size=10).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场名称: `["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]`
            plate_code: 板块代码, 支持地区板块、行业板块、概念板块
            security_code: 股票代码
            size: 数据条数
            verbose: 是否显示日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.market = market
        self.plate_code = plate_code
        self.security_code = security_code
        self.size = size
        self.mapping = CapitalFlowHistoryMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="资金流向历史数据(大盘沪深两市/板块历史数据)", )

    def params_market_code(self) -> Dict:
        """"""
        market_mapping = {
            "沪深两市": "1.000001,0.399001", "沪市": "1.000001", "深市": "0.399001", "创业板": "0.399006", "沪B": "1.000003", "深B": "0.399003"
        }
        if self.market in market_mapping:
            if self.market == "沪深两市":
                secid, secid2 = market_mapping["沪深两市"].split(",")
                return {"secid": secid, "secid2": secid2}
            else:
                return {"secid": market_mapping[self.market]}
        else:
            raise ValueError(
                f'market error {self.market} not in ["沪深两市", "沪市", "深市", "创业板", "沪B", "深B"]')

    def params_plate_code(self) -> Dict:
        """"""
        return {"secid": f"90.{self.plate_code}"}

    def params_security_code(self) -> Dict:
        """"""
        return {"secid": f"0.{self.security_code}"}

    def params_secid(self) -> Dict:
        """"""
        if self.market:
            return self.params_market_code()
        elif self.plate_code:
            return self.params_plate_code()
        elif self.security_code:
            return self.params_security_code()
        else:
            raise ValueError("market_code or plate_code must be set")

    def params(self) -> Dict:
        """
        :return:
        """
        param_sec_id = self.params_secid()
        params = {"lmt": self.size, }
        params.update(param_sec_id)
        return self.base_param(update=params)



from bs4 import BeautifulSoup
from typing import Any, List, Dict, Callable, Optional
from wisecon.types import BaseMapping, BaseRequestData


__all__ = [
    "FundBaseMapping",
    "FundBase",
]


class FundBaseMapping(BaseMapping):
    """字段映射 基金基本信息"""
    columns: Dict = {
        'FullName': '基金全称',
        'ShortName': '基金简称',
        'FundCode': '基金代码',
        'FundType': '基金类型',
        'IssueDate': '发行日期',
        'EstablishmentDate/Capital': '成立日期/规模',
        'AssetSize': '资产规模',
        'ShareSize': '份额规模',
        'FundManager': '基金管理人',
        'Custodian': '基金托管人',
        'FundManagerPerson': '基金经理人',
        'DividendsSinceEstablishment': '成立来分红',
        'ManagementFeeRate': '管理费率',
        'CustodianFeeRate': '托管费率',
        'SalesServiceFeeRate': '销售服务费率',
        'MaximumSubscriptionFeeRate': '最高认购费率',
        'MaximumPurchaseFeeRate': '最高申购费率',
        'MaximumRedemptionFeeRate': '最高赎回费率',
        'PerformanceBenchmark': '业绩比较基准',
        'TrackingIndex': '跟踪标的'
    }


class FundBase(BaseRequestData):
    """查询 基金基本信息"""
    def __init__(
            self,
            fund_code: str,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.fund.fund_value import FundValue

            data = FundValue(fund_code="000001").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            fund_code: 基金代码
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.fund_code = fund_code
        self.mapping = FundBaseMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="text", description="基金基本信息",)

    def base_url(self) -> str:
        """"""
        base_url = f"https://fundf10.eastmoney.com/jbgk_{self.fund_code}.html"
        return base_url

    def clean_content(
            self,
            content: Optional[str] = None,
    ) -> List[Dict]:
        """
        :return:
        """
        soup = BeautifulSoup(content, "html.parser")
        table_info = soup.find_all('table', {'class': 'info w790'})[0]
        td = [item.text for item in table_info.find_all('td')]
        data = dict(zip(list(self.mapping.columns.keys()), td))
        return [data]



from typing import Dict
from pydantic import BaseModel


__all__ = [
    "BaseMapping",
    "APICListMapping",
]


class BaseMapping(BaseModel):
    """"""
    columns: Dict[str, str]

    def reverse(self, mapping_name: str) -> Dict:
        """"""
        if hasattr(self, mapping_name) and isinstance(getattr(self, mapping_name), dict):
            return {v: k for k, v in getattr(self, mapping_name).items()}
        else:
            raise ValueError(f"Mapping {mapping_name} not found in {self.__class__.__name__}")

    def filter_columns(self, columns: Dict) -> Dict:
        """"""
        return {k: v for k, v in columns.items() if v}


class APICListMapping(BaseMapping):
    """"""
    columns: Dict[str, str] = {
        "f12": "code",
        "f13": "prefix_code",
        "f14": "名称",
        "f62": "今日主力净流入(净额)"
    }



import unittest
from wisecon.stock.etf import *


class TestETFGold(unittest.TestCase):
    """"""
    def test_etf_gold_bai(self):
        """"""
        data = ETFGoldHistory(market="ETF白银").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_etf_gold(self):
        """"""
        data = ETFGoldHistory(market="ETF黄金", date="2024-10-25").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_etf_gold_date(self):
        """"""
        data = ETFGoldHistory(market="ETF黄金", start_date="2024-10-25").load()
        print(data.to_frame(chinese_column=True).to_markdown())



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "GDPMapping",
    "GDP",
]


class GDPMapping(BaseMapping):
    """字段映射 GDP"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "DOMESTICL_PRODUCT_BASE": "国内生产总值",
        "FIRST_PRODUCT_BASE": "第一产业绝对值（亿元）",
        "SECOND_PRODUCT_BASE": "第二产业绝对值（亿元）",
        "THIRD_PRODUCT_BASE": "第三产业绝对值（亿元）",
        "SUM_SAME": "国内生产总值同比增长",
        "FIRST_SAME": "第一产业同比增长",
        "SECOND_SAME": "第二产业同比增长",
        "THIRD_SAME": "第三产业同比增长",
    }


class GDP(MacroRequestData):
    """查询 GDP（中国）"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Args:
            size: 最大数据量
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数

        Examples:
            >>> from wisecon.macro import GDP
            >>> data = GDP(size=20).load()
            >>> data.to_frame(chinese_column=True)
        """
        self.size = size
        self.mapping = GDPMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 国内生产总值(GDP)")

    def params(self) -> Dict:
        """"""
        columns = [
            "REPORT_DATE", "TIME", "DOMESTICL_PRODUCT_BASE", "FIRST_PRODUCT_BASE",
            "SECOND_PRODUCT_BASE", "THIRD_PRODUCT_BASE", "SUM_SAME", "FIRST_SAME",
            "SECOND_SAME", "THIRD_SAME"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_GDP",
        }
        return params



from typing import Dict, Optional, List, Literal
from wisecon.types import (
    BaseMapping,
    APICListRequestData,
    APIStockFFlowDayLineRequestData,
    APIUListNPRequestData,
)
from wisecon.utils import time2int


__all__ = [
    "TypeMarket",
    "CapitalFlowMapping",
    "CapitalFlowRequestData",
    "CapitalFlowHistoryBaseMapping",
    "CapitalFlowHistoryRequestData",
    "CapitalFlowCurrentBaseMapping",
    "CapitalFlowCurrentRequestData",
]


TypeMarket = Literal["全部股票", "沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "沪市B股", "深市B股"]


class CapitalFlowMapping(BaseMapping):
    """字段映射 当前资金流量统计"""
    columns: Dict = {
        "f1": "",
        "f2": "最新价",
        "f3": "今日涨跌幅",
        "f12": "代码",
        "f13": "",
        "f14": "名称",
        "f62": "今日主力净流入(净额)",
        "f66": "今日超大单净流入(净额)",
        "f69": "今日超大单净流入(净占比)",
        "f72": "今日大单净流入(净额)",
        "f75": "今日大单净流入(净占比)",
        "f78": "今日中单净流入(净额)",
        "f81": "今日中单净流入(净占比)",
        "f84": "今日小单净流入(净额)",
        "f87": "今日小单净流入(净占比)",
        "f100": "行业名称",
        "f109": "5日涨跌幅",
        "f124": "",
        "f127": "3日涨跌幅",
        "f160": "10日涨跌幅",
        "f164": "5日主力净流入(净额)",
        "f165": "5日主力净流入(净占比)",
        "f166": "5日超大单净流入(净额)",
        "f167": "5日超大单净流入(净占比)",
        "f168": "5日大单净流入(净额)",
        "f169": "5日大单净流入(净占比)",
        "f170": "5日中单净流入(净额)",
        "f171": "5日中单净流入(净占比)",
        "f172": "5日小单净流入(净额)",
        "f173": "5日小单净流入(净占比)",
        "f174": "10日主力净流入(净额)",
        "f175": "10日主力净流入(净占比)",
        "f176": "10日超大单净流入(净额)",
        "f177": "10日超大单净流入(净占比)",
        "f178": "10日大单净流入(净额)",
        "f179": "10日大单净流入(净占比)",
        "f180": "10日中单净流入(净额)",
        "f181": "10日中单净流入(净占比)",
        "f182": "10日小单净流入(净额)",
        "f183": "10日小单净流入(净占比)",
        "f184": "今日主力净流入(占比)",
        "f204": "最大流入股票名称",
        "f205": "最大流入股票代码",
        "f206": "最大流入股票排名",
        "f225": "今日排名",
        "f257": "",
        "f258": "",
        "f259": "",
        "f260": "",
        "f261": "",
        "f263": "5日排名",
        "f264": "10日排名",
        "f265": "所属板块",
        "f267": "3日主力净流入(净额)",
        "f268": "3日主力净流入(净占比)",
        "f269": "3日超大单净流入(净额)",
        "f270": "3日超大单净流入(净占比)",
        "f271": "3日大单净流入(净额)",
        "f272": "3日大单净流入(净占比)",
        "f273": "3日中单净流入(净额)",
        "f274": "3日中单净流入(净占比)",
        "f275": "3日小单净流入(净额)",
        "f276": "3日小单净流入(净占比)",
    }


class CapitalFlowRequestData(APICListRequestData):
    """查询 当前资金流量统计"""
    sort_by: Optional[str]
    days: Optional[Literal[1, 3, 5, 10]]

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {}
        params.update(update)
        return params

    def params_sort_by(self) -> str:
        """"""
        days_mapping = {
            10: "f174",
            5: "f164",
            3: "f267",
            1: "f62",
        }
        if self.sort_by is None:
            if self.days in days_mapping:
                return days_mapping[self.days]
            else:
                return days_mapping[1]
        else:
            return self.sort_by

    def params_fields(self):
        """"""
        fields_mapping = {
            10: "f12,f14,f2,f160,f174,f175,f176,f177,f178,f179,f180,f181,f182,f183,f260,f261,f124,f1,f13",
            5: "f12,f14,f2,f109,f164,f165,f166,f167,f168,f169,f170,f171,f172,f173,f257,f258,f124,f1,f13",
            3: "f12,f14,f2,f127,f267,f268,f269,f270,f271,f272,f273,f274,f275,f276,f257,f258,f124,f1,f13",
            1: "f12,f14,f2,f3,f62,f184,f66,f69,f72,f75,f78,f81,f84,f87,f204,f205,f124,f1,f13",
            "f184": "f2,f3,f12,f13,f14,f62,f184,f225,f165,f263,f109,f175,f264,f160,f100,f124,f265,f1",
        }
        if self.days in fields_mapping:
            return fields_mapping[self.days]
        elif self.sort_by == "f184":
            return fields_mapping[self.sort_by]
        else:
            return fields_mapping[1]


class CapitalFlowHistoryBaseMapping(BaseMapping):
    """字段映射 历史资金流量统计"""
    fields: List = [
        "日期", "主力净流入净额", "小单净流入净额", "中单净流入净额", "大单净流入净额", "超大单净流入净额",
        "主力净流入净占比", "小单净流入净占比", "中单净流入净占比", "大单净流入净占比", "超大单净流入净占比",
        "收盘价", "涨跌幅", "a", "b"
    ]
    columns: Dict = {
        "f1": "日期",
        "f2": "主力净流入净额",
        "f3": "小单净流入净额",
        "f7": "中单净流入净额",
        "f51": "大单净流入净额",
        "f52": "超大单净流入净额",
        "f53": "主力净流入净占比",
        "f54": "小单净流入净占比",
        "f55": "中单净流入净占比",
        "f56": "大单净流入净占比",
        "f57": "超大单净流入净占比",
        "f58": "上证收盘价",
        "f59": "上证涨跌幅",
        "f60": "深证收盘价",
        "f61": "深证涨跌幅",
        "f62": "f62",
        "f63": "f63",
        "f64": "f64",
        "f65": "f65",
    }


class CapitalFlowHistoryRequestData(APIStockFFlowDayLineRequestData):
    """查询 历史资金流量统计"""
    sort_by: Optional[str]
    days: Optional[Literal[1, 3, 5, 10]]

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "lmt": 0,
            "klt": 101,
            "fields1": "f1,f2,f3,f7",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65",
            "_": time2int()
        }
        params.update(update)
        return params


class CapitalFlowCurrentBaseMapping(BaseMapping):
    """字段映射 当前资金流量统计"""
    columns: Dict = {
        "f6": "金额",
        "f62": "今日主力净流入(净额)",
        "f64": "超大单流入",
        "f65": "超大单流出",
        "f66": "今日超大单净流入(净额)",
        "f69": "今日超大单净流入(净占比)",

        "f70": "大单流入",
        "f71": "大单流出",
        "f72": "今日大单净流入(净额)",
        "f75": "今日大单净流入(净占比)",

        "f76": "中单流入",
        "f77": "中单流出",
        "f78": "今日中单净流入(净额)",
        "f81": "今日中单净流入(净占比)",

        "f82": "小单流入",
        "f83": "小单流出",
        "f84": "今日小单净流入(净额)",
        "f87": "今日小单净流入(净占比)",
        "f124": "",
        "f164": "一周主力净流入",
        "f166": "一周超大单净流入",
        "f168": "一周大单净流入",
        "f170": "一周中单净流入",
        "f172": "一周小单净流入",
        "f184": "今日主力净流入(占比)",
        "f252": "一月主力净流入",
        "f253": "一月超大单净流入",
        "f254": "一月大单净流入",
        "f255": "一月中单净流入",
        "f256": "一月小单净流入",
        "f278": "",
        "f279": "",
        "f280": "",
        "f281": "",
        "f282": "",
    }


class CapitalFlowCurrentRequestData(APIUListNPRequestData):
    """查询 当前资金流量统计"""

    def base_param(self, update: Dict) -> Dict:
        """"""
        fields = [
            "f62", "f184", "f66", "f69", "f72", "f75", "f78", "f81",
            "f84", "f87", "f64", "f65", "f70", "f71", "f76", "f77",
            "f82", "f83", "f164", "f166", "f168", "f170", "f172", "f252",
            "f253", "f254", "f255", "f256", "f124", "f6", "f278", "f279",
            "f280", "f281", "f282"
        ]
        params = {
            "fields": ",".join(fields),
            "fltt": 2,
            "_": time2int()
        }
        params.update(update)
        return params



"""
todo: 完成期货、股票、基金全部tools
todo: 增加期货、股票、基金全部Agent Tools
todo: 增加测试、文档
"""

import requests
import pandas as pd
from typing import Literal, Optional, Annotated
from .sina_schema import *


__all__ = [
    "get_stock_kline_data",
    "get_futures_data",
    "get_current_stock_rank",
]


def get_stock_kline_data(
        symbol: Annotated[str, "股票代码", True] = "sh000001",
        scale: Annotated[Optional[Literal[5, 15, 30, 60]], "时间间隔", False] = 5,
        data_len: Annotated[Optional[int], "数据长度", False] = 10,
) -> pd.DataFrame:
    """
    股票历史K-Line数据API

    :param symbol: 股票代码，如：sh000001
    :param scale: 时间间隔min，如：5，15，30，60
    :param data_len: 数据长度，如：10
    :return: DataFrame
    """
    url = f"""https://quotes.sina.cn/cn/api/json_v2.php/CN_MarketDataService.getKLineData?symbol={symbol}&scale={scale}&datalen={data_len}"""
    r = requests.get(url, headers=headers)
    data = r.json()
    data = pd.DataFrame(data)
    return data


def get_futures_data(
        symbol: Annotated[str, "股票代码", True] = "AG2408",
        _type: Annotated[Optional[Literal["1min", "5min", "15min", "30min", "60min", "1day", "5day"]], "时间间隔", False] = "5min",
) -> pd.DataFrame:
    """
    商品期货历史数据API

    :param symbol: 期货代码，如：AG2408
    :param _type: 时间间隔，如：1min，5min，15min，30min，60min，1day，5day
    :return:
    """
    fun_mapping = {
        "getMinLine": ["1min"],
        "getFourDaysLine": ["5day"],
        "getDailyKLine": ["1day"],
        "getFewMinLine": ["5min", "15min", "30min", "60min"],
    }

    fun_name = None
    for fun_name, _types in fun_mapping.items():
        if _type in _types:
            break

    if fun_name is None:
        raise ValueError(f"{_type} is not supported")

    base_url = f"https://stock2.finance.sina.com.cn/futures/api/json.php/InnerFuturesNewService.{fun_name}"
    if fun_name == "getFewMinLine":
        params = {"symbol": symbol, "type": _type}
    else:
        params = {"symbol": symbol}

    r = requests.get(base_url, params=params, headers=headers)
    data = r.json()
    if fun_name == "getFourDaysLine":
        days_data = [pd.DataFrame(day_data) for day_data in data]
        data = pd.concat(days_data, axis=0)
    else:
        data = pd.DataFrame(data)
    return data


def get_current_stock_rank(
        market: Annotated[TypeMarket, "市场", False] = "sh_a",
        sort_by: Annotated[TypeSortBy, "排序字段", False] = "change_percent",
        num: Annotated[Optional[int], "返回条数", False] = 5,
        ascending: Annotated[Optional[Literal[0, 1]], "升序/降序", False] = 0,
) -> pd.DataFrame:
    """
    获取当前股票排名
    :param market: 市场
        如：sh_a: 上证A股; sh_b: 上证B股; sz_a: 深圳A股; sz_b: 深圳B股; hk: 港股, warn: 警示板
    :param sort_by: 排序字段
        如：最新价:trade, 涨跌额:price_change, 涨跌幅:change_percent, 买入:buy, 买出:sell,
        昨日收盘:settlement, 今开:open, 最高:high, 最低:low, 成交量:volume, 成交额:amount
    :param num: 返回数据条数
    :param ascending: 升序/降序，0：降序，1：升序
    :return: Union[DataFrame, List, Markdown]
    """
    if market == "hk":
        func_name = "getHKStockData"
    else:
        func_name = "getHQNodeData"
    base_url = f"https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.{func_name}"
    market = market_mapping.get(market)
    sort_by = sort_by_mapping.get(sort_by)
    params = StockRankParams(page=1, num=num, sort=sort_by, asc=ascending, node=market).model_dump()
    r = requests.get(base_url, params=params, headers=headers)
    data = r.json()
    data = pd.DataFrame(data)
    return data



from .holder import *
from .holder_stock import *
from .holder_change import *
from .holder_stock_analysis import *
from .holder_coop import *
from .free_holder import *
from .free_holder_stock import *
from .free_holder_change import *
from .free_holder_stock_analysis import *
from .free_holder_coop import *



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIAnalystInvest


__all__ = [
    "ResearcherStockMapping",
    "ResearcherStock",
]


class ResearcherStockMapping(BaseMapping):
    """字段映射 分析师持股"""
    columns: Dict = {
        "RATING_DATE": "评级日期",
        "TRADE_MARKET_CODE": "交易市场代码",
        "ANALYST_CODE": "分析师代码",
        "ANALYST_NAME": "分析师姓名",
        "TRADE_DATE": "交易日期",
        "SECURITY_CODE": "证券代码",
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "CHANGE_DATE": "变更日期",
        "RATING_NAME": "评级名称",
        "CLOSE_FORWARD_ADJPRICE": "收盘前调整价格",
        "NEW_PRICE": "最新价格",
        "CURRENT_CHANGE": "当前变动",
        # history
        "BFCHANGE_DATE": "之前变更日期",
        "REASON": "变更原因",
        "CHANGE_RATE": "变更幅度"
    }


class ResearcherStock(APIAnalystInvest):
    """查询 分析师持股"""
    def __init__(
            self,
            analyst_code: Optional[str] = None,
            current: Optional[bool] = True,
            size: Optional[int] = 100,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.analyst import *

            # 查询当前数据
            data = ResearcherStock(analyst_code="11000280036", current=True).load()
            print(data.to_markdown(chinese_column=True))

            # 查询历史数据
            data = ResearcherStock(analyst_code="11000280036", current=False).load()
            print(data.to_markdown(chinese_column=True))
            ```

        Args:
            analyst_code: 分析师代码
            current: 是否当前持股
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.analyst_code = analyst_code
        self.current = current
        self.size = size
        self.mapping = ResearcherStockMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="分析师持股")
        self.conditions = []

    def params_filter(self) -> str:
        """"""
        self.conditions.append(f'(ANALYST_CODE="{self.analyst_code}")')
        return "".join(self.conditions)

    def params_report_name(self) -> str:
        """"""
        if self.current:
            return "RPT_RESEARCHER_NTCSTOCK"
        else:
            return "RPT_RESEARCHER_HISTORYSTOCK"

    def params(self) -> Dict:
        """"""
        params = {
            "sortColumns": "CHANGE_DATE",
            "sortTypes": "-1",
            "pageSize": self.size,
            "reportName": self.params_report_name(),
            "filter": self.params_filter(),
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "ForexLoanMapping",
    "ForexLoan",
]


class ForexLoanMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "时间",
        "TIME": "日期",
        "BASE": "当月(亿美元)",
        "BASE_SAME": "同比增长",
        "BASE_SEQUENTIAL": "环比增长",
        "BASE_ACCUMULATE": "累计(亿美元)",
    }


class ForexLoan(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = ForexLoanMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 外汇贷款数据")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "BASE", "BASE_SAME", "BASE_SEQUENTIAL", "BASE_ACCUMULATE",
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_FOREX_LOAN",
        }
        return self.base_param(update=params)



from pydantic import BaseModel, Field
from typing import Literal

__all__ = [
    "headers",
    # SINA
    "TypeSortBy",
    "TypeMarket",
    "sort_by_mapping",
    "market_mapping",
    # Schema
    "StockRankParams",
]

headers = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.131 Safari/537.36',
}

# sina
TypeSortBy = Literal[
    "trade", "price_change", "change_percent", "buy", "sell", "settlement",
    "open", "high", "low", "volume", "amount",
]

sort_by_mapping = {
    "trade": "trade",                      # 最新价
    "price_change": "pricechange",         # 涨跌额
    "change_percent": "changepercent",     # 涨跌幅
    "buy": "buy",                          # 买入
    "sell": "sell",                        # 买出
    "settlement": "settlement",            # 昨日收盘
    "open": "open",                        # 今开
    "high": "high",                        # 最高
    "low": "low",                          # 最低
    "volume": "volume",                    # 成交量
    "amount": "amount",                    # 成交额
}

TypeMarket = Literal[
    "sh_a", "sh_b", "sz_a", "sz_b", "hk", "warn"
]

market_mapping = {
    "sh_a": "sh_a",        # 上海A
    "sh_b": "sh_b",        # 上海B
    "sz_a": "sz_a",        # 深圳A
    "sz_b": "sz_b",        # 深圳B
    "hk": "qbgg_hk",       # 香港
    "warn": "shfxjs",      # 风险警示
}


class StockRankParams(BaseModel):
    """"""
    page: int = Field(default=1, description="页码")
    num: int = Field(default=5, description="每页条数")
    sort: str = Field(default="change_percent", description="排序字段")
    asc: int = Field(default=0, description="升序")
    node: str = Field(default="sh_a", description="市场")
    symbol: str = Field(default="", description="板块")



"""
Trading View
https://cn.tradingview.com/markets/world-stocks/worlds-largest-companies/

todo: 增加其他数据
todo: 增加Agent Tools测试
todo: 增加文档
"""

import requests
from typing import Dict, Literal, Optional, Annotated
from .base import *


__all__ = [
    "get_stock_technicals",
]


def get_stock_technicals(
        symbol: Annotated[Optional[str], "市场代码", True] = None,
        stock_code: Annotated[Optional[str], "股票代码", True] = None,
        time_period: Annotated[Literal["1min", "5min", "15min", "30min", "60min", "120min", "240min", "1day", "1week", "1month"], "时间周期", True] = "1day",
) -> Dict:
    """
    获取股票技术指标

    :param symbol:
    :param stock_code:
    :param time_period:
    :return:
    """
    url = trading_technicals_mapping.get(time_period)(symbol, stock_code)
    return requests.get(url, headers=headers).json()



from typing import Any, Dict, Callable, Literal, Optional, Annotated
from wisecon.types import BaseMapping
from wisecon.types.request_api.movie import APITVRequestData


__all__ = [
    "TVMapping",
    "TV",
]


class TVMapping(BaseMapping):
    """字段映射 实时电视收视率"""
    columns: Dict = {
        "attentionRate": "关注率",
        "attentionRateDesc": "关注率描述",
        "channelName": "频道名称",
        "marketRate": "市场份额",
        "marketRateDesc": "市场份额描述",
        "programmeName": "节目名称"
    }


class TV(APITVRequestData):
    """查询 实时电视收视率"""
    def __init__(
            self,
            source: Annotated[Literal["央视", "卫视"], "", True] = "央视",
            verbose: Annotated[Optional[bool], "", False] = False,
            logger: Annotated[Optional[Callable], "", False] = None,
            **kwargs: Annotated[Any, "", False],
    ):
        """
        Notes:
            ```python
            from wisecon.movie import *

            # 查询 央视实时电视收视率
            data = TV(source="央视").load()
            data.to_frame(chinese_column=True)

            # 查询 卫视实时电视收视率
            data = TV(source="卫视").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            source: 央视 or 卫视
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.source = source
        self.mapping = TVMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="实时电视收视率",
        )

    def types(self):
        """"""
        mapping = {"央视": 0, "卫视": 1}
        return mapping[self.source]

    def params(self) -> Dict:
        """
        :return:
        """
        params = {"type": self.types()}
        return self.base_param(update=params)



from typing import Any, Dict, Literal, Callable, Optional
from .base import TypeMarket, CapitalFlowRequestData, CapitalFlowMapping


__all__ = [
    "PlateFlowMapping",
    "PlateFlow",
]


class PlateFlowMapping(CapitalFlowMapping):
    """字段映射 市场、板块（行业、概念、地区）资金流向数据"""


class PlateFlow(CapitalFlowRequestData):
    """查询 市场、板块（行业、概念、地区）资金流向数据

    1. 查询市场总体资金流量；返回全部股票当前`1/3/5/10`天的资金动向
    2. 查询全部 `行业、地区、概念` 板块资金流量；返回全部板块下股票当前`1/3/5/10`天的资金动向
    3. 查询指定板块资金流量；返回具体板块下股票当前`1/3/5/10`天的资金动向
    4. 查询不同行业、地区、概念板块下的主力流入流出排名

    Notes:
        ```markdown
        指标定义
        　　- 超大单：大于等于50万股或者100万元的成交单;
        　　- 大单：大于等于10万股或者20万元且小于50万股和100万元的成交单;
        　　- 中单：大于等于2万股或者4万元且小于10万股和20万元的成交单;
        　　- 小单：小于2万股和4万元的成交单;
        　　- 流入：买入成交额;
        　　- 流出：卖出成交额;
        　　- 主力流入：超大单加大单买入成交额之和;
        　　- 主力流出：超大单加大单卖出成交额之和;
        　　- 净额：流入-流出;
        　　- 净比：(流入-流出)/总成交额;
        　　- 5日排名：5日主力净占比排名（指大盘连续交易的5日);
        　　- 5日涨跌：最近5日涨跌幅（指大盘连续交易的5日);
        　　- 10日排名：10日主力净占比排名（指大盘连续交易的10日);
        　　- 10日涨跌：最近10日涨跌幅（指大盘连续交易的10日);
        ```
    """
    def __init__(
            self,
            market: Optional[TypeMarket] = None,
            plate_type: Optional[Literal["行业", "概念", "地区"]] = None,
            plate_code: Optional[str] = None,
            size: Optional[int] = 50,
            sort_by: Optional[str] = None,
            ascending: Optional[bool] = False,
            days: Optional[Literal[1, 3, 5, 10]] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.capital_flow import *

            # 1. 查询市场总体资金流量
            data = PlateFlow(market="沪深A股", days=1).load()
            data.to_frame(chinese_column=True)

            # 2. 查询全部 `行业、地区、概念` 板块资金流量
            data = PlateFlow(plate_type="行业", days=1).load()
            data.to_frame(chinese_column=True)

            # 3. 查询指定板块资金流量
            # 3.1 行业板块
            data = PlateFlow(plate_code="BK1027", days=1).load()
            data.to_frame(chinese_column=True)

            # 3.1 地区板块
            data = PlateFlow(plate_code="BK0158", days=1).load()
            data.to_frame(chinese_column=True)

            # 3.1 概念板块
            data = PlateFlow(plate_code="BK1044", days=1).load()
            data.to_frame(chinese_column=True)

            # 4. 主力排名
            data = PlateFlow(market="全部股票", sort_by="f184").load()
            data.to_frame(chinese_column=True)

            # 4.1 主力流入排名
            PlateFlow(plate_type="行业", days=1, sort_by="f62")
            PlateFlow(plate_type="行业", days=3, sort_by="f267")
            PlateFlow(plate_type="行业", days=5, sort_by="f164")
            PlateFlow(plate_type="行业", days=10, sort_by="f174")

            # 4.2 主力流出排名
            PlateFlow(plate_type="行业", days=1, sort_by="f62", ascending=True)
            PlateFlow(plate_type="行业", days=3, sort_by="f267", ascending=True)
            PlateFlow(plate_type="行业", days=5, sort_by="f164", ascending=True)
            PlateFlow(plate_type="行业", days=10, sort_by="f174", ascending=True)
            ```

        Args:
            market: 市场名称 `["全部股票", "沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "沪市B股", "深市B股"]`
            plate_type: 板块类型, `["行业", "概念", "地区"]`
            plate_code: 板块代码, 支持行业、概念、地区等代码
            days: 统计天数 `1, 3, 5, 10`
            size: 返回数据量
            sort_by: 排序字段名称, 可通过 `PlateFlowMapping` 查询
            verbose: 是否打印日志
            logger: 自定义日志函数
            **kwargs: 其他参数
        """
        self.market = market
        self.plate_type = plate_type
        self.plate_code = plate_code
        self.size = size
        self.sort_by = sort_by
        self.ascending = ascending
        self.days = days
        self.mapping = PlateFlowMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="市场、板块（行业、概念、地区）资金流向数据", )

    def params_market(self) -> str:
        """"""
        market_mapping = {
            "全部股票": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2,m:1+t:2+f:!2,m:1+t:23+f:!2,m:0+t:7+f:!2,m:1+t:3+f:!2",
            "沪深A股": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2,m:1+t:2+f:!2,m:1+t:23+f:!2",
            "沪市A股": "m:1+t:2+f:!2,m:1+t:23+f:!2",
            "科创板": "m:1+t:23+f:!2",
            "深市A股": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2",
            "创业板": "m:0+t:80+f:!2",
            "沪市B股": "m:1+t:3+f:!2",
            "深市B股": "m:0+t:7+f:!2"
        }
        if self.market in market_mapping:
            return market_mapping[self.market]
        else:
            return market_mapping["全部股票"]

    def params_plate_type(self) -> str:
        """"""
        plate_mapping = {
            "行业": "m:90+t:2",
            "概念": "m:90+t:3",
            "地区": "m:90+t:1",
        }
        if self.plate_type in plate_mapping:
            return plate_mapping[self.plate_type]
        else:
            return plate_mapping["行业"]

    def params_plate_code(self) -> str:
        """"""
        return f"b:{self.plate_code}"

    def params_ascending(self) -> int:
        if self.ascending:
            return 0
        else:
            return 1

    def params_fs(self) -> str:
        """"""
        if self.market:
            return self.params_market()
        elif self.plate_type:
            return self.params_plate_type()
        elif self.plate_code:
            return self.params_plate_code()
        else:
            raise ValueError(
                "Invalid parameters, please check the value in the ['params_market', 'plate_type', 'plate_code']")

    def params(self) -> Dict:
        """"""
        sort_by = self.params_sort_by()
        fields = self.params_fields()

        if sort_by not in fields:
            raise ValueError(f"Invalid sort_by value, please check the value in the `fields` attribute. {fields}")

        params = {
            "fid": sort_by,
            "po": self.params_ascending(),
            "pz": self.size,
            "pn": 1,
            "np": 1,
            "fltt": 2,
            "invt": 2,
            "fs": self.params_fs(),
            "fields": fields,
        }
        return params



import unittest
from wisecon.stock.analyst import *


class TestAnalyst(unittest.TestCase):
    """"""

    def test_year_rank(self):
        """"""
        data = AnalystYearRank(year="2024").load()
        print(data.to_markdown(chinese_column=True))

    def test_year_industry_rank(self):
        """"""
        data = AnalystYearRank(year="2024", industry_code="480000").load()
        print(data.to_markdown(chinese_column=True))


class TestAnalystIndex(unittest.TestCase):
    """"""

    def test_analyst_index(self):
        """"""
        data = AnalystIndex(analyst_code="11000280036").load()
        print(data.to_markdown(chinese_column=True))

    def test_analyst_stock_current(self):
        """"""
        data = ResearcherStock(analyst_code="11000280036", current=True).load()
        print(data.to_markdown(chinese_column=True))

    def test_analyst_stock_history(self):
        """"""
        data = ResearcherStock(analyst_code="11000280036", current=False).load()
        print(data.to_markdown(chinese_column=True))




from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIDataRequestData


__all__ = [
    "MarginTradingDailyMapping",
    "MarginTradingDaily",
]


class MarginTradingDailyMapping(BaseMapping):
    """字段映射 融资融券交易总量"""
    columns: Dict = {
        "TRADE_DATE": "交易日期",
        "MARKET": "市场",
        "BOARD_CODE": "板块代码",
        "FIN_BALANCE": "融资余额",
        "LOAN_BALANCE": "借入余额",
        "MARGIN_BALANCE": "保证金余额",
        "FIN_BUY_AMT": "融资买入金额"
    }


class MarginTradingDaily(APIDataRequestData):
    """查询 融资融券交易总量"""
    def __init__(
            self,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.margin import *

            # 查询 融资融券交易总量-市场合计
            data = MarginTradingDaily().load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.mapping = MarginTradingDailyMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="融资融券交易总量")
        self.conditions = []

    def params(self) -> Dict:
        """"""
        columns = [
            "TRADE_DATE", "MARKET", "BOARD_CODE", "FIN_BALANCE",
            "LOAN_BALANCE", "MARGIN_BALANCE", "FIN_BUY_AMT",
        ]
        params = {
            "type": "RPT_MARGIN_MARGINPROFILE",
            "sty": ",".join(columns),
            "p": "1",
            "ps": "5",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "CPIMapping",
    "CPI",
]


class CPIMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "NATIONAL_SAME": "全国同期",
        "NATIONAL_BASE": "全国基期",
        "NATIONAL_SEQUENTIAL": "全国环比",
        "NATIONAL_ACCUMULATE": "全国累计",
        "CITY_SAME": "城市同期",
        "CITY_BASE": "城市基期",
        "CITY_SEQUENTIAL": "城市环比",
        "CITY_ACCUMULATE": "城市累计",
        "RURAL_SAME": "农村同期",
        "RURAL_BASE": "农村基期",
        "RURAL_SEQUENTIAL": "农村环比",
        "RURAL_ACCUMULATE": "农村累计",
    }


class CPI(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = CPIMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国 居民消费价格指数(CPI，上年同月=100)",
        )

    def params(self) -> Dict:
        """"""
        columns = [
            "REPORT_DATE", "TIME", "NATIONAL_SAME", "NATIONAL_BASE", "NATIONAL_SEQUENTIAL",
            "NATIONAL_ACCUMULATE", "CITY_SAME", "CITY_BASE", "CITY_SEQUENTIAL", "CITY_ACCUMULATE",
            "RURAL_SAME", "RURAL_BASE", "RURAL_SEQUENTIAL", "RURAL_ACCUMULATE",
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_CPI",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "IndustryCodeMapping",
    "IndustryCode",
]


class IndustryCodeMapping(BaseMapping):
    """字段映射 行业编码数据"""
    columns: Dict = {
        "INDUSTRY_CODE": "行业编码",
        "INDUSTRY_NAME": "行业名称",
    }


class IndustryCode(APIDataV1RequestData):
    """查询 行业编码数据"""
    def __init__(
            self,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.plate_mapping import *

            # 1. 查询有哪些行业编码
            data = IndustryCode().load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            verbose: 是否打印日志
            logger: 自定义日志函数
            **kwargs: 其他参数
        """
        self.mapping = IndustryCodeMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="行业编码数据", )

    def params(self) -> Dict:
        """"""
        params = {
            "reportName": "RPT_INDUSTRY_SW2021",
            "sortColumns": "INDUSTRY_CODE",
            "sortTypes": 1
        }
        return self.base_param(update=params)



import click
from pydantic import Field
from fastmcp import FastMCP
from typing import Union, Literal
from wisecon.stock.kline import KLine
from wisecon.mcp.validate import *


mcp = FastMCP("Wisecon MCP")


@mcp.tool()
def fetch_stock_data(
    security_code: str = Field(description="security code"),
    period: Literal["1m", "5m", "15m", "30m", "60m", "1D", "1W", "1M"] = Field(default="1D", description="data period"),
    size: int = Field(default=10, description="data size"),
):
    """"""
    data = KLine(security_code=security_code, period=period, size=size).load()
    response = data.to_frame(chinese_column=True)
    return validate_response_data(response)


@click.command()
@click.option("--port", "-p", default=8000, type=int, required=False, help="port")
@click.option("--transport", "-p", default="stdio", type=str, required=False, help="transport")
def stock_mcp_server(
        transport: Literal["stdio", "sse"] = "stdio",
        port: Union[int, str] = None,
) -> None:
    """"""
    if transport == "sse":
        mcp.run(transport=transport, port=port)
    else:
        mcp.run(transport=transport)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "AssetInvestMapping",
    "AssetInvest",
]


class AssetInvestMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "BASE": "当月(亿元)",
        "BASE_SAME": "同比增长",
        "BASE_SEQUENTIAL": "环比增长",
        "BASE_ACCUMULATE": "自年初累计(亿元)",
    }


class AssetInvest(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = AssetInvestMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国 城镇固定资产投资",
        )

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "BASE", "BASE_SAME", "BASE_SEQUENTIAL", "BASE_ACCUMULATE"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_ASSET_INVEST",
        }
        return self.base_param(update=params)



import time
import requests
from pydantic import BaseModel, Field
from typing import Any, Dict, Union, Callable, Optional
from wisecon.utils import LoggerMixin
from wisecon.types.headers import headers
from wisecon.types import ResponseData


__all__ = [
    "ProfitForecast"
]


class ProfitForecastConfig(BaseModel):
    """"""
    size: Optional[int] = Field(default=5, description="数据量")
    code: Optional[Union[str, int]] = Field(default=None, description="股票代码")
    industry: Optional[str] = Field(default=None, description="行业板块")
    district: Optional[str] = Field(default=None, description="地域板块")
    conception: Optional[str] = Field(default=None, description="概念板块")

    def __init__(self,**kwargs):
        super().__init__(**kwargs)

    def _query_condition(self) -> str:
        """"""
        if self.code:
            condition = f'(SECURITY_CODE="{self.code}")'
        elif self.industry:
            condition = f'(INDUSTRY_BOARD="{self.industry}")'
        elif self.district:
            condition = f'(REGION_BOARD="{self.district}")'
        elif self.conception:
            condition = f'(CONCEPTINDEX_BOARD+like"%{self.conception}%")'
        else:
            condition = ""
        return condition

    def params(self) -> Dict:
        """"""
        params = {
            "reportName": "RPT_WEB_RESPREDICT",
            "columns": "WEB_RESPREDICT",
            "pageNumber": "1",
            "pageSize": self.size,
            "sortTypes": "-1",
            "sortColumns": "RATING_ORG_NUM",
            "filter": self._query_condition(),
            "p": "1",
            "pageNo": "1",
            "pageNum": "1",
            "_": str(int(time.time() * 1E3)),
        }
        return params


class ProfitForecast(LoggerMixin):
    """"""
    def __init__(
            self,
            query_config: Optional[ProfitForecastConfig] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """"""
        if query_config is None:
            self.query_config = ProfitForecastConfig.model_validate(kwargs)
        else:
            self.query_config = query_config
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

    def load_data(self, ) -> ResponseData:
        """"""
        base_url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
        params = self.query_config.to_params()
        response = requests.get(base_url, params=params, headers=headers.headers)
        metadata = response.json().get("result", {})
        data = metadata.pop("data")
        self.update_metadata(metadata)
        self._logger(msg=f"[{__class__.__name__}] Find {len(data)} reports.", color="green")
        return ResponseData(data=data, metadata=metadata)

    def update_metadata(self, metadata: Dict):
        """"""
        metadata.update({
            "description": "机构盈利预测",
            "columns": {
                "SECUCODE": "股票代码",
                "SECURITY_CODE": "证券代码",
                "SECURITY_NAME_ABBR": "证券简称",
                "RATING_ORG_NUM": "评级机构数量",
                "RATING_BUY_NUM": "买入评级数量",
                "RATING_ADD_NUM": "增持评级数量",
                "RATING_NEUTRAL_NUM": "中性评级数量",
                "RATING_REDUCE_NUM": "减持评级数量",
                "RATING_SALE_NUM": "卖出评级数量",
                "YEAR1": "第一年",
                "YEAR_MARK1": "第一年标记",
                "EPS1": "第一年每股收益",
                "YEAR2": "第二年",
                "YEAR_MARK2": "第二年标记",
                "EPS2": "第二年每股收益",
                "YEAR3": "第三年",
                "YEAR_MARK3": "第三年标记",
                "EPS3": "第三年每股收益",
                "YEAR4": "第四年",
                "YEAR_MARK4": "第四年标记",
                "EPS4": "第四年每股收益",
                "INDUSTRY_BOARD": "行业板块",
                "INDUSTRY_BOARD_SZM": "深证行业板块",
                "CONCEPTINDEX_BOARD": "概念指数板块",
                "CONCEPTINDEX_BOARD_SZM": "深证概念指数板块",
                "REGION_BOARD": "区域板块",
                "REGION_BOARD_SZM": "深证区域板块",
                "MARKET_BOARD": "市场板块",
                "DEC_AIMPRICEMAX": "目标最高价",
                "DEC_AIMPRICEMIN": "目标最低价",
                "RATING_LONG_NUM": "长期评级数量",
            }
        })



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "FaithIndexMapping",
    "FaithIndex",
]


class FaithIndexMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "CONSUMERS_FAITH_INDEX": "消费者信心指数",
        "FAITH_INDEX_SAME": "消费者信心指数（同比）",
        "FAITH_INDEX_SEQUENTIAL": "消费者信心指数（环比）",
        "CONSUMERS_ASTIS_INDEX": "消费者满意指数",
        "ASTIS_INDEX_SAME": "消费者满意指数（同比）",
        "ASTIS_INDEX_SEQUENTIAL": "消费者满意指数（环比）",
        "CONSUMERS_EXPECT_INDEX": "消费者预期指数",
        "EXPECT_INDEX_SAME": "消费者预期指数（同比）",
        "EXPECT_INDEX_SEQUENTIAL": "消费者预期指数（环比）",
    }


class FaithIndex(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = FaithIndexMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 消费者信心指数")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "CONSUMERS_FAITH_INDEX", "FAITH_INDEX_SAME", "FAITH_INDEX_SEQUENTIAL",
            "CONSUMERS_ASTIS_INDEX", "ASTIS_INDEX_SAME", "ASTIS_INDEX_SEQUENTIAL", "CONSUMERS_EXPECT_INDEX",
            "EXPECT_INDEX_SAME", "EXPECT_INDEX_SEQUENTIAL"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_FAITH_INDEX",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "GoldCurrencyMapping",
    "GoldCurrency",
]


class GoldCurrencyMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "GOLD_RESERVES": "黄金储备(亿美元)",
        "GOLD_RESERVES_SAME": "黄金储备(同比)",
        "GOLD_RESERVES_SEQUENTIAL": "黄金储备(环比)",
        "FOREX": "国家外汇储备(亿美元)",
        "FOREX_SAME": "国家外汇储备(同比)",
        "FOREX_SEQUENTIAL": "国家外汇储备(环比)",
    }


class GoldCurrency(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = GoldCurrencyMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 外汇和黄金储备")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "GOLD_RESERVES", "GOLD_RESERVES_SAME", "GOLD_RESERVES_SEQUENTIAL",
            "FOREX", "FOREX_SAME", "FOREX_SEQUENTIAL"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_GOLD_CURRENCY",
        }
        return self.base_param(update=params)



import unittest
from wisecon.types.columns import StockFeatures


class TestColumns(unittest.TestCase):
    def test_stock_features(self):
        stock_features = StockFeatures()
        stock_features._mapping(columns=["f31"])





from typing import Any, Dict, Optional, Literal, Callable
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "DepartmentStatisticMapping",
    "DepartmentStatistic",
]


class DepartmentStatisticMapping(BaseMapping):
    """字段映射 营业部统计"""
    columns: Dict = {
        "OPERATEDEPT_CODE": "营业部代码",
        "OPERATEDEPT_NAME": "营业部名称",
        "STATISTICSCYCLE": "统计周期",
        "AMOUNT": "金额",
        "SALES_ONLIST_TIMES": "上榜成交次数",
        "ACT_BUY": "实际买入",
        "TOTAL_BUYER_SALESTIMES": "总买方成交次数",
        "ACT_SELL": "实际卖出",
        "TOTAL_SELLER_SALESTIMES": "总卖方成交次数",
        "OPERATEDEPT_CODE_OLD": "旧营业部代码",
        "ORG_NAME_ABBR": "组织名称简称"
    }


class DepartmentStatistic(APIDataV1RequestData):
    """查询 营业部统计"""

    def __init__(
            self,
            statistics_cycle: Literal["1m", "3m", "6m", "12m"] = "1m",
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.ranking import *

            data = DepartmentStatistic(statistics_cycle="1m").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            statistics_cycle: 统计周期, 可选值: 1m, 3m, 6m, 12m, 默认值: 1m
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.statistics_cycle = statistics_cycle
        self.size = size
        self.mapping = DepartmentStatisticMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="营业部统计",)
        self.conditions = []

    def params_statistics_cycle(self):
        """"""
        cycle_mapping = {"1m": "01", "3m": "02", "6m": "03", "12m": "04"}
        self.conditions.append(f'(STATISTICSCYCLE="{cycle_mapping.get(self.statistics_cycle, "01")}")')

    def params_filter(self) -> str:
        """"""
        self.params_statistics_cycle()
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "AMOUNT,OPERATEDEPT_CODE",
            "sortTypes": "-1,1",
            "reportName": "RPT_OPERATEDEPT_LIST_STATISTICS",
        }
        return self.base_param(update=params)



import unittest
from wisecon.agent import *
from wisecon.movie import *
from wisecon.stock.etf import *
from wisecon.stock.market import *
from wisecon.stock.financial import *
from wisecon.stock.kline import *
from wisecon.stock.tick import *


class TestToolCall(unittest.TestCase):
    """"""
    def setUp(self):
        base_url = "https://open.bigmodel.cn/api/paas/v4"
        tools = Tools(tools=[
            LastDayMarketSummary,
            KLine,
            KlineMin,
            Tick,
            StockBalance,
            ETFGoldHistory,
            TV, Movie])
        self.agent = BaseAgent(
            model="glm-4-plus", base_url=base_url, tools=tools,
            api_key_name="ZHIPU_API_KEY")

    def test_gold_history(self):
        """"""
        completion = self.agent.chat(prompt="查询2023年12月1日的ETF白银历史数据")

    def test_tv(self):
        """"""
        completion = self.agent.chat(prompt="查询目前央视最受喜欢的节目")

    def test_movie(self):
        """"""
        completion = self.agent.chat(prompt="现在最受新欢的电影是哪部")

    def test_balance(self):
        """"""
        completion = self.agent.chat(prompt="2024年第三季度末纺织服装行业资产负债表前五名是多少？")

    def test_kline(self):
        """"""
        # completion = self.agent.chat(prompt="当前300069股票的价格是多少？")
        completion = self.agent.chat(prompt="当前300069股票最近5分钟的最低价价格是多少？")

    def test_kline_min(self):
        """"""
        completion = self.agent.chat(prompt="当前300069股票最近1分钟的最低价价格是多少？")

    def test_tick(self):
        """"""
        completion = self.agent.chat(prompt="当前300069股票的tick数据？")

    # def




from typing import Iterable


__all__ = [
    "is_notebook",
    "tqdm_progress_bar",
]


def is_notebook():
    try:
        from IPython import get_ipython
        return 'IPKernelApp' in get_ipython().config
    except Exception:
        return False


def tqdm_progress_bar(iterable: Iterable, **kwargs):
    """"""
    if is_notebook():
        from tqdm.notebook import tqdm
        return tqdm(iterable, **kwargs)
    else:
        from tqdm import tqdm
        return tqdm(iterable, **kwargs)



import unittest
from wisecon.stock.valuation import *


class TestMarketValuation(unittest.TestCase):
    """"""
    def test_market_valuation(self):
        """"""
        data = MarketValuation(market="沪深两市", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_market_date(self):
        data = MarketValuation(market="沪深两市", start_date="2024-10-01", end_date="2024-10-20", size=50).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_market_valuation_loop(self):
        """"""
        for market in ["沪深两市", "沪市主板", "科创板", "深市主板", "创业板",]:
            print(f"Market: {market}")
            data = MarketValuation(market=market, size=5).load()
            print(data.to_frame(chinese_column=True).to_markdown())


class TestIndustryValuation(unittest.TestCase):
    """"""
    def test_valuation_date(self):
        """"""
        data = IndustryValuation(date="2024-10-10", verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_industry_code(self):
        """"""
        data = IndustryValuation(start_date="2024-10-01", end_date="2024-10-20", limit=50, industry_code="016017").load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_save_data(self):
        """"""
        data = IndustryValuation(date="2024-10-10", industry_code=None).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestStockValuation(unittest.TestCase):
    """"""
    def test_valuation_date(self):
        """"""
        data = StockValuation(date="2024-09-30").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_industry_code(self):
        """"""
        data = StockValuation(date="2024-09-30", industry_code="016023").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_code(self):
        """"""
        data = StockValuation(start_data="2024-08-30", code="000059").load()
        print(data.to_frame(chinese_column=True).to_markdown())



import unittest
from wisecon.stock.main_holder import *


class TestFundHolder(unittest.TestCase):
    def test_fund_holder_list(self):
        data = FundHolderList(holder_code="007751", date="2024-09-30").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_fund_main_holder(self):
        """"""
        data = MainHolder(holder="基金", date="2024-09-30").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_fund_holder_history(self):
        """"""
        data = StockFundHolderHistory(security_code="000001", date="2024-09-30").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_holder(self):
        data = StockHolder(security_code="603350", holder="基金", date="2024-09-30").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_holder_collect(self):
        data = StockHolderCollect(security_code="603350", date="2024-09-30").load()
        print(data.to_frame(chinese_column=True).to_markdown())



from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import BaseMapping, APICListRequestData


__all__ = [
    "CurrentMarketMapping",
    "CurrentMarket",
]


TypeMarket = Literal[
    "沪深京A股",
    "上证A股", "注册制上证A股", "核准制上证A股",
    "深证A股", "注册制深证A股", "核准制深证A股",
    "创业板", "注册制创业板", "核准制创业板",
    "北证A股", "新股", "科创板", "沪股通", "深股通", "B股",
    "上证AB股比价", "深证AB股比价",
    "风险警示板", "风险警示板(SH)", "风险警示板(SZ)", "风险警示板(科创)", "风险警示板(创业)", "两网及退市",
    "ETF", "LOF", "封闭基金",
]


class CurrentMarketMapping(BaseMapping):
    """字段映射 `ETF/LOF/Stock`当前市场行情"""
    columns: Dict = {
        "f1": "",
        "f2": "最新价",
        "f3": "涨跌幅",
        "f4": "涨跌额",
        "f5": "成交量",
        "f6": "成交额",
        "f7": "振幅",
        "f8": "换手率",
        "f9": "市盈率",
        "f10": "量比",
        "f12": "证券代码",
        "f13": "",
        "f14": "证券名称",
        "f15": "最高价",
        "f16": "最低价",
        "f17": "开盘价",
        "f18": "昨收",
        "f20": "总市值",
        "f21": "流通市值",
        "f23": "市净率",
        "f24": "60日涨跌幅",
        "f25": "今年涨跌幅",
        "f26": "上市时间",
        "f22": "涨速",
        "f11": "",
        "f62": "",
        "f128": "",
        "f136": "",
        "f115": "",
        "f152": "",
        "f140": "",
        "f141": "",
        "f195": "B股最新价",
        "f196": "",
        "f197": "B股涨跌幅",
        "f199": "比价(A/B)",
        "f200": "",
        "f201": "B股代码",
        "f202": "",
        "f203": "B股名称",
    }


class CurrentMarket(APICListRequestData):
    """查询 `ETF/LOF/Stock`当前市场行情"""
    def __init__(
            self,
            market: TypeMarket = "ETF",
            sort_by: Optional[str] = "f3",
            page_size: Optional[int] = 20,
            page_number: Optional[int] = 1,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.market import *

            # 查询ETF市场当前行情
            data = CurrentMarket(market="ETF",).load()
            data.to_frame(chinese_column=True)

            # 查询LOF市场当前行情
            data = CurrentMarket(market="LOF",).load()
            data.to_frame(chinese_column=True)

            # 查询封闭基金当前行情
            data = CurrentMarket(market="封闭基金",).load()
            data.to_frame(chinese_column=True)

            # 查询沪深京A股当前行情
            data = CurrentMarket(market="沪深京A股",).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场类型: ```
                [
                    "沪深京A股",
                    "上证A股", "注册制上证A股", "核准制上证A股",
                    "深证A股", "注册制深证A股", "核准制深证A股",
                    "创业板", "注册制创业板", "核准制创业板",
                    "北证A股", "新股", "科创板", "沪股通", "深股通", "B股",
                    "上证AB股比价", "深证AB股比价",
                    "风险警示板", "风险警示板(SH)", "风险警示板(SZ)", "风险警示板(科创)", "风险警示板(创业)", "两网及退市",
                    "ETF", "LOF", "封闭基金",
                ]```
            sort_by: 排序字段
            page_size: 每页数据量
            page_number: 页码
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.market = market
        self.sort_by = sort_by
        self.page_size = page_size
        self.page_number = page_number
        self.mapping = CurrentMarketMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="`ETF/LOF/Stock`当前市场行情")

    def params_market(self) -> str:
        """"""
        market_mapping = {
            "沪深京A股": "m:0+t:6,m:0+t:80,m:1+t:2,m:1+t:23,m:0+t:81+s:2048",
            "上证A股": "m:1+t:2,m:1+t:23",
            "注册制上证A股": "m:1+t:2+s:131072,m:1+t:23+s:131072",
            "核准制上证A股": "m:1+t:2+s:524288,m:1+t:23+s:524288",
            "深证A股": "m:0+t:6,m:0+t:80",
            "注册制深证A股": "m:0+t:6+s:131072,m:0+t:80+s:131072",
            "核准制深证A股": "m:0+t:6+s:524288,m:0+t:80+s:524288",
            "北证A股": "m:0+t:81+s:2048",
            "新股": "m:0+f:8,m:1+f:8",
            "创业板": "m:0+t:80",
            "注册制创业板": "m:0+t:80+s:131072",
            "核准制创业板": "m:0+t:80+s:!131072",
            "科创板": "m:1+t:23",
            "沪股通": "b:BK0707",
            "深股通": "b:BK0804",
            "B股": "m:0+t:7,m:1+t:3",
            "上证AB股比价": "m:1+b:BK0498",
            "深证AB股比价": "m:0+b:BK0498",
            "风险警示板": "m:0+f:4,m:1+f:4",
            "风险警示板(SH)": "m:1+f:4",
            "风险警示板(SZ)": "m:0+f:4",
            "风险警示板(科创)": "m:1+t:23+f:4",
            "风险警示板(创业)": "m:0+t:80+f:4",
            "两网及退市": "m:0+s:3",
            "ETF": "b:MK0021,b:MK0022,b:MK0023,b:MK0024,b:MK0827",
            "LOF": "b:MK0404,b:MK0405,b:MK0406,b:MK0407",
            "封闭基金": "e:19",
        }
        return market_mapping[self.market]

    def params(self) -> Dict:
        """"""
        columns_base = [
            "f1", "f2", "f3", "f4", "f5", "f6", "f7", "f8", "f9", "f10",
            "f12", "f13", "f14", "f15", "f16", "f17", "f18", "f20",
            "f21", "f23", "f24", "f25", "f22", "f11",
            "f62", "f128", "f136", "f115", "f152"
        ]
        columns_compare = [
            "f201", "f202", "f203", "f196", "f197", "f199", "f195", "f200"
        ]

        if self.market in ["上证AB股比价"]:
            fields = ",".join(columns_base + columns_compare)
        else:
            fields = ",".join(columns_base)
        params = {
            "pn": self.page_number,
            "pz": self.page_size,
            "fid": self.sort_by,
            "fs": self.params_market(),
            "fields": fields,
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "PMIMapping",
    "PMI",
]


class PMIMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "MAKE_INDEX": "制造业指数",
        "MAKE_SAME": "制造业同比增长",
        "NMAKE_INDEX": "非制造业指数",
        "NMAKE_SAME": "非制造业同比增长",
    }


class PMI(MacroRequestData):
    """采购经理人指数(PMI)"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.verbose = verbose
        self.mapping = PMIMapping()
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国 采购经理人指数(PMI)",
        )

    def params(self) -> Dict:
        """"""
        columns = [
            "REPORT_DATE", "TIME", "MAKE_INDEX", "MAKE_SAME", "NMAKE_INDEX",
            "NMAKE_SAME"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_PMI",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Annotated, Optional
from wisecon.types import BaseMapping, APIStockKlineWithSSE
from wisecon.types.columns import StockFeatures


__all__ = [
    "TickMapping",
    "Tick",
]


class TickMapping(BaseMapping):
    """字段映射 股票实时tick数据"""
    columns: Dict = StockFeatures().tick_columns()


class Tick(APIStockKlineWithSSE):
    """查询 股票实时tick数据"""
    def __init__(
            self,
            code: Annotated[Optional[str], "", False] = None,
            verbose: Annotated[Optional[bool], "", False] = False,
            logger: Annotated[Optional[Callable], "", False] = None,
            **kwargs: Annotated[Any, "", False],
    ):
        """
        股票实时tick数据

        Notes:
            ```python
            from wisecon.stock.tick import Tick

            # 查询证券代码301618的实时tick数据
            data = Tick(code="301618").load()
            print(data.to_frame(chinese_column=True))
            ```

        Args:
            code: 证券代码
            verbose: 是否打印日志
            logger: 日志打印函数
            **kwargs: 其他参数
        """
        self.code = code
        self.mapping = TickMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="股票实时tick数据")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "f11", "f12", "f13", "f14", "f15", "f16", "f17", "f18", "f19", "f20",
            "f31", "f32", "f33", "f34", "f35", "f36", "f37", "f38", "f39", "f40",
            "f191", "f192", "f531",
        ]
        params = {
            "fields": ",".join(columns),
            "mpi": 1000,
            "invt": 2,
            "fltt": 1,
            "secid": f"0.{self.code}",
            "dect": 1,
            "wbp2u": "|0|0|0|web"
        }
        return params
